package com.liangjian11.ums.config;

import com.baomidou.mybatisplus.MybatisConfiguration;
import com.baomidou.mybatisplus.plugins.PaginationInterceptor;
import com.baomidou.mybatisplus.spring.MybatisSqlSessionFactoryBean;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.PlatformTransactionManager;

import javax.annotation.Resource;
import javax.sql.DataSource;

/**
 * @Auther: FanJiaKai
 * @Date: 2019/3/5 13:36
 * @Description:
 */
@Configuration
@MapperScan(basePackages = {"com.liangjian11.ums.dao.foo"}, sqlSessionTemplateRef = "fooSqlSessionTemplate")
public class FooDataSourceConfig {

    /**
     * 数据源配置对象
     * Primary 表示默认的对象，Autowire可注入，不是默认的得明确名称注入
     * @return
     */
    @Bean
    @ConfigurationProperties("foo.datasource")
    public DataSourceProperties fooDataSourceProperties() {
        return new DataSourceProperties();
    }

    /**
     * 数据源对象
     * @return
     */
    @Bean(name = "datasource.auth")
//    @Primary
    public DataSource fooDataSource() {
        DataSourceProperties dataSourceProperties = fooDataSourceProperties();
        return dataSourceProperties.initializeDataSourceBuilder().build();
    }

    /**
     * 配置事物
     * @param fooDataSource
     * @return
     */
    @Bean
    @Resource
    public PlatformTransactionManager fooManager(@Qualifier("datasource.auth")DataSource fooDataSource){
        return new DataSourceTransactionManager(fooDataSource);
    }

    /**
     * 配置Mapper路径
     * @param dataSource
     * @return
     * @throws Exception
     */
    @Bean
    public  SqlSessionFactory fooSqlSessionFactory(@Qualifier("datasource.auth") DataSource dataSource) throws Exception  {
        MybatisSqlSessionFactoryBean bean  =  new  MybatisSqlSessionFactoryBean();
        //spring boot多数据源mybatis-plus的baseMapper的里面的方法无法使用 SqlSessionFactoryBean改为MybatisSqlSessionFactoryBean即可，
        // 否则baseMapper自带的方法不能访问，但能访问*Mapper.xml中定义的方法
        bean.setDataSource(dataSource);
//        MybatisConfiguration configuration = new MybatisConfiguration();
//        configuration.setMapUnderscoreToCamelCase(true);
        //添加XML目录
        ResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        try{
            bean.setMapperLocations(resolver.getResources("classpath*:mapper/**Mapper.xml"));//application.yml文件中已经配置，无需再配置
            return  bean.getObject();
        }catch(Exception  e){
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    @Bean
    public SqlSessionTemplate fooSqlSessionTemplate(@Qualifier("fooSqlSessionFactory") SqlSessionFactory sqlSessionFactory) throws Exception {
        SqlSessionTemplate template = new SqlSessionTemplate(sqlSessionFactory); // 使用上面配置的Factory
        return template;
    }

    /**
     * mybatis-plus分页插件<br>
     * 文档：http://mp.baomidou.com<br>
     */
    @Bean
    public PaginationInterceptor paginationInterceptor() {
        PaginationInterceptor page = new PaginationInterceptor();
        page.setDialectType("mssql");
        return page;
    }
}
