package com.liangjian11.ums.controller;

import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.exception.AuthException;
import org.apache.shiro.ShiroException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.web.ErrorProperties;
import org.springframework.boot.autoconfigure.web.servlet.error.BasicErrorController;
import org.springframework.boot.web.servlet.error.DefaultErrorAttributes;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;


@ControllerAdvice
public class GlobalExceptionController extends BasicErrorController {//BasicErrorController 继承处理过滤器异常

    private static final Logger log = LoggerFactory.getLogger(GlobalExceptionController.class);

    public GlobalExceptionController() {
        super(new DefaultErrorAttributes(), new ErrorProperties());
    }

    @RequestMapping(produces = {MediaType.APPLICATION_JSON_VALUE})
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    public ResponseEntity error(HttpServletRequest request) {
        Map<String, Object> body = getErrorAttributes(request, isIncludeStackTrace(request, MediaType.ALL));
        HttpStatus statusIn = getStatus(request);
//        //自定义的错误信息类
//        //status.value():错误代码，
//        //body.get("message").toString()错误信息
////        R ret = R.error(status.value(), body.get("message").toString());
//        //TokenException Filter抛出的自定义错误类
//
////        if (!Strings.isNullOrEmpty((String) body.get("exception")) && body.get("exception").equals(AuthException.class.getName())) {
////            body.put("status", HttpStatus.UNAUTHORIZED.value());
////            status = HttpStatus.UNAUTHORIZED;
////        }
        HttpStatus status = HttpStatus.OK;
        if(statusIn.toString() == HttpStatus.NOT_FOUND.toString())
            status = HttpStatus.NOT_FOUND;
        ResultInfo resultInfo = ResultUtil.createFail(105);
        resultInfo.setResult(body);
        return new ResponseEntity(resultInfo, status);
    }

    // 捕捉shiro的异常
//    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(ShiroException.class)
    @ResponseBody
    public ResultInfo handleShiroException(ShiroException e) {
        this.logException(e);
        return ResultUtil.createFail(105);
    }

    // 捕捉UnauthorizedException
//    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(AuthException.class)
    @ResponseBody
    public ResultInfo handleAuthException(AuthException e) {
        this.logException(e);
        return ResultUtil.createFail(e.getCode());
    }
//
//    // 捕捉UnauthorizedException
//    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
//    @ExceptionHandler(AuthException.class)
//    @ResponseBody
//    public ResultInfo handleInAuthException(AuthException e) {
//        this.logException(e);
//        return ResultUtil.createFail(e.getCode());
//    }

    // 捕捉其他所有异常
    @ExceptionHandler(Throwable.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ResponseBody
    public ResultInfo globalException(Throwable ex) {
        this.logException(ex);
        return ResultUtil.createFail(500);
    }

    private void logException(Throwable e) {
        log.error("There is an exception = {}", e.getMessage(), e);
    }
}
