package com.liangjian11.ums.controller.orgnization;


import com.baomidou.mybatisplus.plugins.Page;
import com.liangjian11.ums.model.InterfaceResource;
import com.liangjian11.ums.model.common.PageExt;
import com.liangjian11.ums.model.organization.Orgdoc;
import com.liangjian11.ums.service.ClientService;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.service.organization.OrgdocService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * (Orgdoc)表控制层
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Api(value = "机构控制器", tags = "机构控制器")
@RestController
@RequestMapping("/ums/orgdoc")
public class OrgdocController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private OrgdocService orgdocService;

    @Autowired
    private ClientService clientService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @return 所有数据
     */
    @ApiOperation(value = "条件分页查询Orgdoc")
    @PostMapping("/searchByCondition")
    public ResultInfo searchByCondition(@ApiParam(value = "Page对象", required = true) @RequestBody Page<Orgdoc> page) {
        Page<Orgdoc> orgdocPage = this.orgdocService.selectPage(page);
        orgdocPage.setTotal(this.orgdocService.selectCount(null));
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "ok",orgdocPage);
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "根据id查询 Orgdoc")
    @GetMapping("/{id}")
    public ResultInfo selectOne(@ApiParam(value = "ID", required = true) @PathVariable("id") Integer id) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "ok",this.orgdocService.selectById(id));
    }

    /**
     * 新增数据
     *
     * @param orgdoc 实体对象
     * @return 新增结果
     */
    @PostMapping("/add")
    @ApiOperation(value = "新增Orgdoc")
    public ResultInfo insert(@ApiParam(value = "Orgdoc", required = true) @RequestBody Orgdoc orgdoc) {
        try {
            orgdocService.add(orgdoc);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "保存失败 异常："+e.getMessage());
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "ok");
    }

    /**
     * 修改数据
     *
     * @param orgdoc 实体对象
     * @return 修改结果
     */
    @PutMapping("/update")
    @ApiOperation(value = "根据id修改Orgdoc")
    public ResultInfo update(@ApiParam(value = "Orgdoc", required = true) @RequestBody Orgdoc orgdoc) {
        try {
            if(this.orgdocService.updateByOrgId(orgdoc)){
                return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "更新成功",orgdoc);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "更新失败 异常："+e.getMessage());
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "更新失败");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @DeleteMapping("/delete")
    @ApiOperation(value = "删除Orgdoc")
    public ResultInfo delete(@ApiParam(value = "Orgdoc", required = true) @RequestParam("idList") List<Long> idList) {
        if(this.orgdocService.deleteBatchIds(idList)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "删除成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "删除失败");
    }

    @PostMapping("/query")
    @ApiOperation("查询机构信息")
    public ResultInfo query(@ApiParam(value = "Page对象", required = true) @RequestBody PageExt<InterfaceResource> page) {

        Page pageData = this.orgdocService.selectOrgdocPage(page);

        return ResultUtil.createSuccess(200,pageData);
    }
}