package com.liangjian11.ums.controller.orgnization;


import com.baomidou.mybatisplus.plugins.Page;
import com.liangjian11.ums.model.organization.OrgdocResource;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.service.organization.OrgdocResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * (Orgdoc)表控制层
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Api(value = "机构接口控制器", tags = "机构接口控制器")
@RestController
@RequestMapping("/ums/orgdocResource")
public class OrgdocResourceController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private OrgdocResourceService orgdocResourceService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @return 所有数据
     */
    @ApiOperation(value = "条件分页查询Orgdoc")
    @PostMapping("/searchByCondition")
    public ResultInfo selectAll(@ApiParam(value = "Page对象", required = true) @RequestBody Page<OrgdocResource> page) {
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.orgdocResourceService.selectPage(page));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "根据id查询 OrgdocResource")
    @GetMapping("/{id}")
    public ResultInfo selectOne(@ApiParam(value = "ID", required = true) @PathVariable("id") Integer id) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.orgdocResourceService.selectById(id));
    }


    /**
     * 通过主键查询单条数据
     *
     * @param orgId 主键
     * @return 单条数据
     */
    @ApiOperation(value = "根据orgId查询 Resource")
    @GetMapping("/selectByOrgId")
    public ResultInfo selectByOrgId(@ApiParam(value = "ID", required = true) @RequestParam("orgId") Integer orgId) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.orgdocResourceService.selectByOrgId(orgId));
    }

    /**
     * 新增数据
     *
     * @param orgdocResource 实体对象
     * @return 新增结果
     */
    @PostMapping("/add")
    @ApiOperation(value = "新增Orgdoc")
    public ResultInfo insert(@ApiParam(value = "OrgdocResource", required = true) @RequestBody OrgdocResource orgdocResource) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.orgdocResourceService.insert(orgdocResource));
    }


    /**
     * 批量新增数据
     *
     * @param orgdocResource 实体对象
     * @return 新增结果
     */
    @PostMapping("/addBatch")
    @ApiOperation(value = "批量新增Orgdoc")
    public ResultInfo insertBatch(@ApiParam(value = "OrgdocResource", required = true) @RequestBody List<OrgdocResource> orgdocResource) {
        this.orgdocResourceService.addBatch(orgdocResource);
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "插入成功");
    }

    /**
     * 修改数据
     *
     * @param orgdocResource 实体对象
     * @return 修改结果
     */
    @PutMapping("/update")
    @ApiOperation(value = "根据id修改Orgdoc")
    public ResultInfo update(@ApiParam(value = "OrgdocResource", required = true) @RequestBody OrgdocResource orgdocResource) {
        if(this.orgdocResourceService.updateById(orgdocResource)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "更新成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "更新失败");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @DeleteMapping("/delete")
    @ApiOperation(value = "删除Orgdoc")
    public ResultInfo delete(@ApiParam(value = "OrgdocResource", required = true) @RequestParam("idList") List<Long> idList) {
        if(this.orgdocResourceService.deleteBatchIds(idList)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "删除成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "删除失败");
    }

    /**
     * 删除数据
     *
     * @param orgdocResource 主键结合
     * @return 删除结果
     */
    @DeleteMapping("/deleteBatch")
    @ApiOperation(value = "根据OrgId批量删除OrgdocResource")
    public ResultInfo deleteBatch(@ApiParam(value = "OrgdocResource", required = true) @RequestBody List<OrgdocResource> orgdocResource) {
        this.orgdocResourceService.deleteBatch(orgdocResource);
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "删除成功");
    }
}