package com.liangjian11.ums.controller.orgnization;


import com.baomidou.mybatisplus.plugins.Page;
import com.liangjian11.ums.model.InterfaceResource;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.service.InterfaceResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 权限表(Resource)表控制层
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Api(value = "接口控制器", tags = "接口控制器")
@RestController
@RequestMapping("/resource")
public class ResourceController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private InterfaceResourceService resourceService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @return 所有数据
     */
    @ApiOperation(value = "条件分页查询Resource")
    @PostMapping("/searchByCondition")
    public ResultInfo selectAll(@ApiParam(value = "Page对象", required = true) @RequestBody Page<InterfaceResource> page) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "", this.resourceService.selectPage(page));
    }

    /**
     * 分页查询所有数据
     *
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有Resource")
    @GetMapping("/all")
    public ResultInfo selectAll() {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "", this.resourceService.selectAll());
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "根据id查询 Resource")
    @GetMapping("/{id}")
    public ResultInfo selectOne(@ApiParam(value = "ID", required = true) @PathVariable("id") Integer id) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "", this.resourceService.selectById(id));
    }

    /**
     * 新增数据
     *
     * @param resource 实体对象
     * @return 新增结果
     */
    @PostMapping("/add")
    @ApiOperation(value = "新增Resource")
    public ResultInfo insert(@ApiParam(value = "Resource", required = true) @RequestBody InterfaceResource resource) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "", this.resourceService.insert(resource));
    }

    /**
     * 修改数据
     *
     * @param resource 实体对象
     * @return 修改结果
     */
    @PutMapping("/update")
    @ApiOperation(value = "根据id修改Resource")
    public ResultInfo update(@ApiParam(value = "Resource", required = true) @RequestBody InterfaceResource resource) {
        if(this.resourceService.updateById(resource)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "更新成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "更新失败");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @DeleteMapping("/delete")
    @ApiOperation(value = "删除Resource")
    public ResultInfo delete(@ApiParam(value = "Resource", required = true) @RequestParam("idList") List<Long> idList) {
        if(this.resourceService.deleteBatchIds(idList)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "删除成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "删除失败");
    }
}