package com.liangjian11.ums.controller.orgnization;


import com.baomidou.mybatisplus.plugins.Page;
import com.liangjian11.ums.model.common.PageExt;
import com.liangjian11.ums.model.organization.ResourceType;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.service.organization.ResourceTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 接口类型表(ResourceType)表控制层
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Api(value = "接口类型控制器", tags = "接口类型控制器")
@RestController
@RequestMapping("/ums/resourceType")
public class ResourceTypeController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private ResourceTypeService resourceTypeService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @return 所有数据
     */
    @ApiOperation(value = "条件分页查询接口类型")
    @PostMapping("/searchByCondition")
    public ResultInfo selectAll(@ApiParam(value = "Page对象", required = true) @RequestBody Page<ResourceType> page) {
        ResultInfo resultInfo = new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "", this.resourceTypeService.selectPage(page));
        resultInfo.setTotalCount(this.resourceTypeService.selectCount(null));
        return resultInfo;
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "根据id查询接口类型")
    @GetMapping("/{id}")
    public ResultInfo selectOne(@ApiParam(value = "ID", required = true) @PathVariable("id") Integer id) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.resourceTypeService.selectById(id));
    }


    /**
     * 查询所有接口类型数据
     *
     * @return 单条数据
     */
    @ApiOperation(value = "查询所有接口类型")
    @GetMapping("/all")
    public ResultInfo selectAll() {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.resourceTypeService.selectAll());
    }

    /**
     * 新增数据
     *
     * @param resourceType 实体对象
     * @return 新增结果
     */
    @PostMapping("/add")
    @ApiOperation(value = "新增接口类型")
    public ResultInfo insert(@ApiParam(value = "ResourceType", required = true) @RequestBody ResourceType resourceType) {
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "",this.resourceTypeService.insert(resourceType));
    }

    /**
     * 修改数据
     *接口类型
     * @param resourceType 实体对象
     * @return 修改结果
     */
    @PutMapping("/update")
    @ApiOperation(value = "根据id修改ResourceType")
    public ResultInfo update(@ApiParam(value = "ResourceType", required = true) @RequestBody ResourceType resourceType) {
        if(this.resourceTypeService.updateById(resourceType)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "更新成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "更新失败");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @DeleteMapping("/delete")
    @ApiOperation(value = "删除接口类型")
    public ResultInfo delete(@ApiParam(value = "ResourceType", required = true) @RequestParam("idList") List<Long> idList) {
        if(this.resourceTypeService.deleteBatchIds(idList)){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "删除成功");
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_FAIL, 200, "删除失败");
    }

    /**
     * 查询所有接口类型数据
     *
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有接口类型")
    @GetMapping("/selectAllAsJson")
    public ResultInfo selectAllAsJson() {
        return ResultUtil.createSuccess(200,  this.resourceTypeService.selectAllAsJson());
    }

    @PostMapping("/query")
    @ApiOperation("条件分页查询接口类型信息")
    public ResultInfo query(@ApiParam(value = "Page对象", required = true) @RequestBody PageExt<ResourceType> page) {

        Page pageData = this.resourceTypeService.selectResourceTypePage(page);

        return ResultUtil.createSuccess(200,pageData);
    }


}