package com.liangjian11.ums.controller.shirooauth2;


import com.auth0.jwt.JWT;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.Application;
import com.liangjian11.ums.model.Client;
import com.liangjian11.ums.service.ClientService;
import com.liangjian11.ums.service.IApplicationService;
import com.liangjian11.ums.service.IOAuthService;
import com.liangjian11.ums.service.IUserService;
import com.liangjian11.ums.utils.JWTUtil;
import com.liangjian11.ums.utils.RSAUtil;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.web.shiro.JWTGenerator;
import com.liangjian11.ums.web.shiro.JWTPassGenerator;
import com.liangjian11.ums.web.shiro.JWTRSA256Generator;
import com.liangjian11.ums.web.shiro.JWTRefreshGenerator;
import com.liangjian11.ums.constant.Constants;
import com.liangjian11.ums.controller.shiro.dto.auth.AuthTokenParams;
import com.liangjian11.ums.exception.AuthException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.oltu.oauth2.as.issuer.MD5Generator;
import org.apache.oltu.oauth2.as.issuer.OAuthIssuerImpl;
import org.apache.oltu.oauth2.as.issuer.ValueGenerator;
import org.apache.oltu.oauth2.as.request.OAuthTokenRequest;
import org.apache.oltu.oauth2.as.response.OAuthASResponse;
import org.apache.oltu.oauth2.common.OAuth;
import org.apache.oltu.oauth2.common.error.OAuthError;
import org.apache.oltu.oauth2.common.exception.OAuthProblemException;
import org.apache.oltu.oauth2.common.exception.OAuthSystemException;
import org.apache.oltu.oauth2.common.message.OAuthResponse;
import org.apache.oltu.oauth2.common.message.types.GrantType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 令牌生成
 * */
@RestController
@Slf4j
@RequestMapping("/ums/oauth2")
public class AccessTokenController {

    @Autowired
    private IOAuthService oAuthService;

    @Autowired
    private IUserService iUserService;

    @Autowired
    private IApplicationService iApplicationService;

    @Autowired
    private ClientService clientService;

    @Value("${account.security.privatekey}")
    private String privateKey;

//    @Value("${jwt.token.active.time}")
//    private static long activeTime =  2 * 60 * 60 * 1000;//毫秒

    @GetMapping(value = "/")
    public String defaultPath(Model model) {
        return "hello";
    }

    private synchronized ResponseEntity check(OAuthTokenRequest oauthRequest) throws OAuthSystemException {
        //检查提交的客户端ID是否正确
        if(!oAuthService.checkClientId(oauthRequest.getClientId())){
            log.error("获取accessToken时，客户端ID错误 client=" + oauthRequest.getClientId());
            OAuthResponse response = OAuthResponse
                    .errorResponse(HttpServletResponse.SC_BAD_REQUEST)
                    .setError(OAuthError.TokenResponse.INVALID_CLIENT)
                    .setErrorDescription(Constants.INVALID_CLIENT_DESCRIPTION)
                    .buildJSONMessage();
            return new ResponseEntity(response.getBody(), HttpStatus.valueOf(response.getResponseStatus()));
        }

        // 检查客户端安全KEY是否正确
        if(!oAuthService.checkClientSecret(oauthRequest.getClientSecret())){
            log.error("ClientSecret不合法 client_secret="+oauthRequest.getClientSecret());
            OAuthResponse response = OAuthResponse
                    .errorResponse(HttpServletResponse.SC_UNAUTHORIZED)
                    .setError(OAuthError.TokenResponse.UNAUTHORIZED_CLIENT)
                    .setErrorDescription(Constants.INVALID_CLIENT_DESCRIPTION)
                    .buildJSONMessage();
            return new ResponseEntity(response.getBody(), HttpStatus.valueOf(response.getResponseStatus()));
        }

        return null;
    }

    @RequestMapping(value = "/accessToken")
    public HttpEntity token(HttpServletRequest request) throws Exception {
        try{
            // 转为OAuth请求
            OAuthTokenRequest oauthRequest = new OAuthTokenRequest(request);
            AuthTokenParams authTokenParms = new AuthTokenParams(request);

            ResponseEntity responseEntity = this.check(oauthRequest);
            if(responseEntity!=null){
                return responseEntity;
            }

            String authCode = oauthRequest.getParam(OAuth.OAUTH_CODE);
            ValueGenerator valueGenerator = new MD5Generator();
            String tokenValue = "";

            /**
             * 此处只校验 AUTHORIZATION_CODE 类型，其他的还有PASSWORD 、REFRESH_TOKEN 和 CLIENT_CREDENTIALS
             * 具体查看 {@link GrantType}
             * */
            if(oauthRequest.getParam(OAuth.OAUTH_GRANT_TYPE).equals(GrantType.AUTHORIZATION_CODE.toString())){
                if(!oAuthService.checkAuthCode(authCode)){
                    OAuthResponse response = OAuthResponse
                            .errorResponse(HttpServletResponse.SC_BAD_REQUEST)
                            .setError(OAuthError.TokenResponse.INVALID_GRANT)
                            .setErrorDescription(Constants.INVALID_CODE_DESCRIPTION)
                            .buildJSONMessage();
                    return new ResponseEntity(response.getBody(), HttpStatus.valueOf(response.getResponseStatus()));
                }

//                grantType = grantType.replace("authorizationCode","authorization_code");
                tokenValue = oAuthService.getUsernameByAuthCode(authCode);
            }else if(oauthRequest.getParam(OAuth.OAUTH_GRANT_TYPE).equals(GrantType.PASSWORD.toString())){
                //根据用户名密码访问资源
                if ( Objects.nonNull(authTokenParms) && StringUtils.isNoneBlank(authTokenParms.getApplicationKey())
                        && StringUtils.isNoneBlank(authTokenParms.getUserName())
                        && StringUtils.isNoneBlank(authTokenParms.getPassword()) ) {

                    Account user = this.iUserService.getUserByUserName(authTokenParms.getUserName());
                    if (Objects.isNull(user)) {
                        throw new AuthException("账号已被冻结或者账号不存在", 101);
                    }
                    user.setExtra("{\"WxAppIds\":[\"wx4f8c9fef3896ad71\",\"wx4f8c9fef3896ad71\"]}");


                    Application application = this.iApplicationService.getApplicationByAppkey(authTokenParms.getApplicationKey());
                    if (Objects.isNull(application)) {
                        throw new AuthException("Application " + authTokenParms.getApplicationKey() + " doesn't exist!", 120);
                    }

                    //添加RSA解密
                    if(!authTokenParms.getPassword().equals(RSAUtil.decrypt(user.getPassword(),privateKey))){
                        throw new AuthException("Password error!", 114);
                    }
//                    if (!authTokenParms.getPassword().equals(user.getPassword())) {
//                        throw new AuthException("Password error!", 114);
//                    }

                    tokenValue = authTokenParms.getUserName() + "_" + authTokenParms.getPassword() + "_"+authTokenParms.getApplicationKey()
                            + "_" + user.getRoleKey() + "_" + user.getDeptCode() + "_"+user.getOrgKey() + "_"+user.getExtra()
                            + "_" + oAuthService.getExpireIn() + "_" + GrantType.PASSWORD.toString();
                    valueGenerator = new JWTPassGenerator(tokenValue);
                }else
                    throw new AuthException("Params invalid!", 504);
            }else if(oauthRequest.getParam(OAuth.OAUTH_GRANT_TYPE).equals(GrantType.REFRESH_TOKEN.toString())){
                //根据refresh_token返回刷新的access_token令牌 先简单返回access_token
                tokenValue = oAuthService.getUsernameByAccessToken(request.getHeader("Sso-Token"));
                valueGenerator = new JWTRefreshGenerator(oauthRequest.getRefreshToken());
            }else if(oauthRequest.getParam(OAuth.OAUTH_GRANT_TYPE).equals(GrantType.CLIENT_CREDENTIALS.toString())){
                Client client = clientService.findByClientId(oauthRequest.getClientId());
                String clientPrivSecret = client.getClientPrivSecret();

                String grantType = GrantType.CLIENT_CREDENTIALS.toString()
                                .replace("client_credentials","clientCredentials");

                //RSA加密方式生成token
                tokenValue = oauthRequest.getClientId() + "_"
                        + client.getClientSecret() + "-"
                        + clientPrivSecret + "_"
                        +authTokenParms.getApplicationKey() + "_"
                        + oAuthService.getExpireIn()+ "_"
                        +(grantType);

                /**判断是否存在token 如果已经有并且没有过期 不需要再生成token 返回
                 * */
                if(authTokenParms.getAutoRefresh() == true) {
                    String token = oAuthService.getAccessToken(oauthRequest.getClientId());
                    if(!StringUtils.isEmpty(token)){
                        DecodedJWT jwt = JWTUtil.decodeToken(token);
                        Date expiresAt = JWTUtil.getExpiresAt(jwt,token);
                        String tokenData = oAuthService.getAccessUserByToken(token);

                        if ( expiresAt.after(new Date()) && tokenData.equals(tokenValue) ) {

                            ResultInfo resultInfo = ResultUtil.createSuccess(200, null);
    //            resultInfo.setResult(response.getBody());
                            Map<String, String> data = new HashMap<String, String>();
                            data.put("access_token", token);
                            data.put("expires_in", String.valueOf(oAuthService.getExpireIn()));
                            resultInfo.setResult(data);

                            log.info("服务器缓存的accessToken="+token);
                            // 生成OAuth响应
                            OAuthResponse response = OAuthASResponse
                                    .tokenResponse(HttpServletResponse.SC_OK)
                                    .setAccessToken(token)
                                    .setExpiresIn(String.valueOf(oAuthService.getExpireIn()))
                                    .buildJSONMessage();

                            // 根据OAuthResponse 生成ResponseEntity
                            return new ResponseEntity(resultInfo,HttpStatus.valueOf(response.getResponseStatus()));
                        }

                    }
                }


                valueGenerator = new JWTRSA256Generator(tokenValue);
                //直接返回access_token令牌
            }

            // 生成Access Token
            OAuthIssuerImpl oAuthIssuer = new OAuthIssuerImpl(valueGenerator);
            String accessToken = oAuthIssuer.accessToken();
            log.info("服务器生成的accessToken="+accessToken);
            oAuthService.addAccessToken(accessToken,tokenValue);

            // 移除code，每个code只能使用一次
            if(oauthRequest.getParam(OAuth.OAUTH_GRANT_TYPE).equals(GrantType.AUTHORIZATION_CODE.toString())) {
                String username = oAuthService.removeAuthCode(authCode);
                log.info("服务器移除auth_code=" + authCode + " username=" + username);
            }

            // 生成OAuth响应
            OAuthResponse response = OAuthASResponse
                    .tokenResponse(HttpServletResponse.SC_OK)
                    .setAccessToken(accessToken)
                    .setExpiresIn(String.valueOf(oAuthService.getExpireIn()))
                    .buildJSONMessage();

            ResultInfo resultInfo = ResultUtil.createSuccess(200, null);
//            resultInfo.setResult(response.getBody());
            Map<String,String> data = new HashMap<String,String>();
            data.put("access_token",accessToken);
            data.put("expires_in",String.valueOf(oAuthService.getExpireIn()));
            resultInfo.setResult(data);

            // 根据OAuthResponse 生成ResponseEntity
            return new ResponseEntity(resultInfo,HttpStatus.valueOf(response.getResponseStatus()));

        }catch (OAuthProblemException e){
            log.error("获取accessToken发生异常e=",e);
            // 构建错误响应
            OAuthResponse response = OAuthASResponse
                    .errorResponse(HttpServletResponse.SC_BAD_REQUEST).error(e)
                    .buildJSONMessage();
            return new ResponseEntity(response.getBody(),HttpStatus.valueOf(response.getResponseStatus()));
        }
    }

}
