package com.liangjian11.ums.controller.shirooauth2;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.liangjian11.ums.controller.shiro.dto.auth.JudgePerMissionparams;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.common.Menu;
import com.liangjian11.ums.service.IApplicationService;
import com.liangjian11.ums.service.IUserService;
import com.liangjian11.ums.utils.JWTUtil;
import com.liangjian11.ums.utils.content.Config;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.exception.AuthException;
import com.liangjian11.ums.service.InterfaceResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresAuthentication;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;


@RestController
@Api(tags = "基于Shiro鉴权服务", description = "API接口")
@RequestMapping("/ums/oauth2")
public class ShiroAuthController extends BaseController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ShiroAuthController.class);

    @Autowired
    private IUserService iUserService;

    @Autowired
    private InterfaceResourceService iResourceService;

    @Autowired
    private IApplicationService iApplicationService;

    @Value("${jwt.token.active.time}")
    private static long activeTime = 5 * 60 * 1000;



    @PostMapping(value = "/judge/permission")
    @ApiOperation(value = "判断当前用户是否有权限")
    @ApiImplicitParams({
            @ApiImplicitParam(name = Config.JWT_CUSTOMER_TOKEN_NAME, value = "认证token", required = true, dataType = "string", paramType = "header")
    })
    @RequiresAuthentication
    public ResultInfo judgePermission(JudgePerMissionparams judgePerMissionparams) {
//        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
//        HttpServletRequest request = requestAttributes.getRequest();

        ResultInfo resultInfo = ResultUtil.createSuccess(200, null);

        if (Objects.nonNull(judgePerMissionparams) && StringUtils.isNotBlank(judgePerMissionparams.getUrlAddress())) {

            boolean isAllowed = SecurityUtils.getSubject().isPermitted(judgePerMissionparams.getUrlAddress());
            Map<String, Boolean> stringBooleanMap = new HashMap<>();
            stringBooleanMap.put("isAllowed", isAllowed);
            resultInfo.setResult(stringBooleanMap);

        } else {
            throw new AuthException("Params invalid!", 504);
        }

        return resultInfo;
    }

    @PostMapping(value = "/list/permission")
    @ApiOperation(value = "获取权限列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = Config.JWT_CUSTOMER_TOKEN_NAME, value = "认证token", required = true, dataType = "string", paramType = "header")
    })
    @RequiresAuthentication
    public ResultInfo listPermission(@RequestHeader(Config.JWT_CUSTOMER_TOKEN_NAME) String token,@RequestParam(value="types",defaultValue="-1",required=true) Integer types) {
        DecodedJWT decodedJWT = JWTUtil.decodeToken(token);
        String grantType = decodedJWT.getType();

        ResultInfo resultInfo = ResultUtil.createSuccess(200, null);
        //if(types == -1) //-1代表所有权限

        if (SecurityUtils.getSubject().isAuthenticated()) {
            String applicationKey = JWTUtil.getApplicationKey(token);
            String username = JWTUtil.getUsername(token);
            Account user = this.iUserService.getUserByUserName(username);
            if (SecurityUtils.getSubject().hasRole(Config.SUPER_ADMIN_ROLE)) {
                resultInfo.setResult(Menu.Convert(this.iResourceService.getResourcesByAppKey(applicationKey,types)));
            } else {
                resultInfo.setResult(Menu.Convert(this.iResourceService.getResourcesByUserAndAppKey(user, applicationKey,types,grantType)));
            }
        } else {
            throw new AuthException("Params invalid!", 504);
        }

        return resultInfo;
    }

    @GetMapping(value = "/401")
    @ApiIgnore
    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    public ResultInfo unauthorized() {
        return ResultUtil.createFail(119);
    }

}
