package com.liangjian11.ums.controller.shirooauth2;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.service.IUserService;
import com.liangjian11.ums.utils.JWTUtil;
import com.liangjian11.ums.utils.content.Config;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.exception.AuthException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.oltu.oauth2.common.message.types.GrantType;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresAuthentication;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;


@RestController
@Api(tags = "基于Shiro用户服务", description = "API接口")
@RequestMapping("/ums/oauth2")
public class ShiroUserController extends BaseController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ShiroUserController.class);

    @Autowired
    private IUserService iUserService;

    @GetMapping(value = "/judge/token")
    @ApiOperation(value = "判断当前用户是否已认证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = Config.JWT_CUSTOMER_TOKEN_NAME, value = "认证token", required = true, dataType = "string", paramType = "header")
    })
    @RequiresAuthentication
    public ResultInfo judgToken(@RequestHeader(value = Config.JWT_CUSTOMER_TOKEN_NAME) String token) {

        ResultInfo resultInfo = ResultUtil.createSuccess(200, null);

        if (StringUtils.isNotBlank(token)) {
            String grantType = JWTUtil.getGrantType(token);

            grantType = grantType.replace("clientCredentials","client_credentials");
            grantType = grantType.replace("authorizationCode","authorization_code");
            if(grantType.equals(GrantType.PASSWORD.toString()) || grantType.equals(GrantType.AUTHORIZATION_CODE.toString()) ) {
                boolean isAllowed = SecurityUtils.getSubject().isAuthenticated();

                if (isAllowed) {
                    String payloadValue = JWTUtil.getPayloadValue(token);
                    String userName = JWTUtil.getUsername(token);
                    Account user = this.iUserService.getUserByUserName(userName);
                    user.setPassword(null);

                    Map<String, Object> data = JSONObject.parseObject(JSON.toJSONString(user));
                    data.put("payload", payloadValue);
                    String grantType1 = JWTUtil.getGrantType(token);
                    data.put(Config.JWT_GRANT_TYPE, grantType1);

                    resultInfo.setResult(data);
                } else {
                    throw new AuthException("Token invalid!", 119);
                }
            }else if( grantType.equals(GrantType.CLIENT_CREDENTIALS.toString()) ){
                String payloadValue = JWTUtil.getPayloadValue(token);

                Map<String, Object> data = new HashMap<>();
                data.put("payload", payloadValue);
                String grantType1 = JWTUtil.getGrantType(token);
                data.put(Config.JWT_GRANT_TYPE, grantType1);

                resultInfo.setResult(data);
            }else{
                throw new AuthException("Params invalid!", 504);
            }
        } else {
            throw new AuthException("Params invalid!", 504);
        }

        return resultInfo;
    }
}