package com.liangjian11.ums.controller.ums;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.Role;
import com.liangjian11.ums.model.UserRole;
import com.liangjian11.ums.model.common.FiltersData;
import com.liangjian11.ums.model.common.FiltersUtils;
import com.liangjian11.ums.service.IRoleService;
import com.liangjian11.ums.service.IUserRoleService;
import com.liangjian11.ums.service.IUserService;
import com.liangjian11.ums.utils.RSAUtil;
import com.liangjian11.ums.utils.content.Config;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.web.shiro.JWTRSA256Generator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import java.util.*;


@Api(value = "用户控制器", tags = "用户控制器")
@RestController
@RequestMapping("/ums/account")
public class AccountController extends BaseController {

    /**
     * 密码公钥
     */
    @Value("${account.security.publickey}")
    private String publicKey;

    /**
     * 密码私钥
     */
    @Value("${account.security.privatekey}")
    private String privateKey;

    /**
     * 用户服务
     */
    @Autowired
    private IUserService userService;
    /**
     * 角色服务
     */
    @Autowired
    private IRoleService roleService;
    /**
     * 用户角色服务
     */
    @Autowired
    private IUserRoleService userRoleService;



    @ApiOperation(value = "条件分页查询Account")
    @PostMapping("/queryAccounts")
    public ResultInfo queryAccounts(@ApiParam(value = "FiltersData对象", required = true) @RequestBody FiltersData filtersData) throws Exception {
        if (filtersData.getPageIndex() <= 0 && filtersData.getPageSize() <= 0) {
            return ResultUtil.createFail(new IllegalArgumentException("参数违法,PageIndex、PageSize不能为空或<=0"));
        }
        int start = (filtersData.getPageIndex() - 1) * filtersData.getPageSize();
        int pageSize = filtersData.getPageSize();
        String filterSql = "where 1=1 ";
        if(!CollectionUtils.isEmpty(filtersData.getFilters())){
            for (FiltersUtils filter : filtersData.getFilters()) {
                if ("nickName".equals(filter.getField())) {
                    filter.setValue(filter.getValue().replace("'", ""));
                }
            }
            filterSql = filtersData.getFilterString();
        }
        String filterSortSql = filterSql;
        if(!StringUtils.isEmpty(filtersData.getSortField()) && filtersData.getSortDirection()!=null){
            filterSortSql += filtersData.getSortString();
        }else{
            filterSortSql += " order by creationDate desc ";
        }
        List<Account> accountList = userService.getUserByCondition(filterSortSql, start, pageSize);
        for (Account account : accountList) {
            if(account.getPassword().length()>=117){
                account.setPassword(RSAUtil.decrypt(account.getPassword(),privateKey));
            }
        }
        int count = 0;
        if (!CollectionUtils.isEmpty(accountList)) {
            count = userService.selectCountByCondition(filterSql);
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "操作成功", "查询所有用户成功", accountList, count);
    }


    @ApiOperation(value = "添加Account")
    @PostMapping("/addAccount")
    public ResultInfo addAccount(@ApiParam(value = "Account对象", required = true) @RequestBody Account account) throws Exception {
        if (StringUtils.isEmpty(account.getNickName()) || StringUtils.isEmpty(account.getLoginId()) || StringUtils.isEmpty(account.getPassword())) {
            return ResultUtil.createFail(new IllegalArgumentException("创建用户失败,缺少必要参数"));
        }
        if(userService.selectCount(new EntityWrapper<Account>().eq("loginId",account.getLoginId()))>0){
            return ResultUtil.createFail(new IllegalArgumentException("创建用户失败,登录名已存在"));
        }
        if(userService.selectCount(new EntityWrapper<Account>().eq("nickName",account.getNickName()))>0){
            return ResultUtil.createFail(new IllegalArgumentException("创建用户失败,昵称已存在"));
        }
        account.setId(UUID.randomUUID().toString());
        //密码RSA加密
        String encodedPassword = RSAUtil.encrypt(account.getPassword(),publicKey);
        account.setPassword(encodedPassword);
        account.setPassword1(encodedPassword);
        Date date = new Date();
        account.setCreateDate(date);
        account.setLastModified(date);
        account.setCreationDate(date);
        //0：注销?，2:激活 -4:删除?
        account.setRecordState(2);
        //0:禁用 1:启用 -3:垃圾桶 -4:删除
        account.setStatus(1);
        account.setAccountCode(Integer.parseInt(userService.selectMaxAccountCode())+1+"");
        account.setCreateByUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
        account.setUpdateByUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
        return userService.insertUserWithRole(account);
    }


    @ApiOperation(value = "更新Account")
    @PostMapping("/editAccount")
    public ResultInfo editAccount(@ApiParam(value = "Account对象", required = true) @RequestBody Account account) throws Exception {
        if (StringUtils.isEmpty(account.getId())) {
            return ResultUtil.createFail(new IllegalArgumentException("更新用户失败,缺少必要参数"));
        }
        Account oldAccount = userService.selectOne(new EntityWrapper<Account>().eq("id", account.getId()));
        if(oldAccount == null){
            return ResultUtil.createFail(new IllegalArgumentException("更新用户失败,无此账户"));
        }
        if(!StringUtils.isEmpty(oldAccount.getLoginId())&&!oldAccount.getLoginId().equals( account.getLoginId())){
            if(userService.selectCount(new EntityWrapper<Account>().eq("loginId",account.getLoginId()))>0){
                return ResultUtil.createFail(new IllegalArgumentException("更新用户失败,登录名已存在"));
            }
        }
        if(!StringUtils.isEmpty(oldAccount.getNickName())&&!oldAccount.getNickName().equals(account.getNickName())){
            if(userService.selectCount(new EntityWrapper<Account>().eq("nickName",account.getNickName()))>0){
                return ResultUtil.createFail(new IllegalArgumentException("更新用户失败,昵称已存在"));
            }
        }
        List<Integer> recordStatusList = new ArrayList<>();
        recordStatusList.add(0);
        recordStatusList.add(2);
        if(account.getRecordState()!=null & !recordStatusList.contains(account.getRecordState())){
            return ResultUtil.createFail(new IllegalArgumentException("冻结用户失败,状态不合法"));
        }
        //密码MD5加密
        if(!StringUtils.isEmpty(account.getPassword())){
            account.setPassword(RSAUtil.encrypt(account.getPassword(),publicKey));
            account.setPassword1(account.getPassword());
        }
        account.setLastModified(new Date());
        account.setUpdateByUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
        return userService.updateUserWithRole(account);
    }


    @ApiOperation(value = "删除Account")
    @PostMapping("/deleteAccount")
    public ResultInfo deleteAccount(@ApiParam(value = "Account对象", required = true) @RequestBody Account account) {
        if(!StringUtils.isEmpty(account.getId())){
            return userService.deleteUserWithRole(account.getId());
        }else{
            return ResultUtil.createFail(new IllegalArgumentException("参数违法,账号ID为空"));
        }
    }

//    @ApiOperation(value = "根据AccountID查询关联角色列表")
//    @PostMapping("/queryRolesByUser")
//    public ResultInfo queryRolesByUser(@ApiParam(value = "Account对象", required = true) @RequestBody Account account) {
//        if(!StringUtils.isEmpty(account.getId())){
//            List<Role> roleList = roleService.listRoleByUser(account.getId());
//            int count = 0;
//            if(!CollectionUtils.isEmpty(roleList)){
//                count=roleList.size();
//            }
//            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS,200,"操作成功","查询该用户所有角色成功",roleList,count);
//        }
//        return ResultUtil.createFail(new IllegalArgumentException("参数违法,账号ID为空"));
//    }
//
//
//    @ApiOperation(value = "Account添加单个Role")
//    @PostMapping("/addUserRole")
//    public ResultInfo addUserRole(@ApiParam(value = "UserRole对象", required = true) @RequestBody UserRole userRole) {
//        if(!StringUtils.isEmpty(userRole.getRoleId()) && !StringUtils.isEmpty(userRole.getAccountId())){
//            userRole.setId(UUID.randomUUID().toString());
//            Date date = new Date();
//            userRole.setCreationTime(date);
//            userRole.setCreateByUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
//            userRole.setLastModified(date);
//            userRole.setLastModifiedUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
//            userRole.setStatus(1);
//            userRoleService.insert(userRole);
//            return ResultUtil.createSuccess(200,"关联角色成功");
//        }else{
//            return ResultUtil.createFail(new IllegalArgumentException("关联角色失败,账号ID或角色ID为空"));
//        }
//    }
//
//
//    @ApiOperation(value = "Account Role状态修改")
//    @PostMapping("/editUserRoleStatus")
//    public ResultInfo editUserRoleStatus(@ApiParam(value = "Account、Role键值对数组", required = true) @RequestBody UserRole userRole) {
//        List<Integer> statusType = new ArrayList<>();
//        statusType.add(0);
//        statusType.add(1);
//        statusType.add(-3);
//        statusType.add(-4);
//        if (!StringUtils.isEmpty(userRole.getId()) && userRole.getStatus() != null && statusType.contains(userRole.getStatus())) {
//            userRole.setLastModified(new Date());
//            userRole.setLastModifiedUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
//            EntityWrapper<UserRole> entityWrapper = new EntityWrapper<>();
//            entityWrapper.eq("id", userRole.getId());
//            if (userRoleService.selectCount(entityWrapper) > 0) {
//                 boolean result  = userRoleService.updateById(userRole);
//                 return ResultUtil.createSuccess(200, "更新用户角色状态成功");
//            } else {
//                return ResultUtil.createFail(new IllegalArgumentException("更新用户角色状态失败，用户角色关系不存在’"));
//            }
//        } else {
//            return ResultUtil.createFail(new IllegalArgumentException("更新用户角色状态失败，用户角色关系ID为空或Status无效’"));
//        }
//    }
//
//    @ApiOperation(value = "Account批量添加Role")
//    @PostMapping("/addUserRoles")
//    public ResultInfo addUserRoles(@ApiParam(value = "UserRole数组", required = true) @RequestBody List<UserRole> userRoleList) {
//        for (UserRole userRole : userRoleList) {
//            if(StringUtils.isEmpty(userRole.getRoleId()) || StringUtils.isEmpty(userRole.getAccountId())){
//                return ResultUtil.createFail(new IllegalArgumentException("添加用户角色状态失败，用户ID或角色ID为空"));
//            }else{
//                userRole.setId(UUID.randomUUID().toString());
//                Date date = new Date();
//                userRole.setCreationTime(date);
//                userRole.setCreateByUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
//                userRole.setLastModified(date);
//                userRole.setLastModifiedUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
//                userRole.setStatus(1);
//            }
//        }
//        return userRoleService.insertBatchUserRoles(userRoleList);
//    }

}
