package com.liangjian11.ums.controller.ums;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.model.InterfaceResource;
import com.liangjian11.ums.model.Role;
import com.liangjian11.ums.model.common.FiltersData;
import com.liangjian11.ums.model.common.FiltersUtils;
import com.liangjian11.ums.model.common.Option;
import com.liangjian11.ums.model.common.PageExt;
import com.liangjian11.ums.service.IOAuthService;
import com.liangjian11.ums.service.IRoleService;
import com.liangjian11.ums.utils.content.Config;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import com.liangjian11.ums.service.InterfaceResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.oltu.oauth2.common.exception.OAuthSystemException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 权限表(Resource)表控制层
 *
 * @author pzc
 * @since 2020-04-14 14:12:53
 */
@Api(value = "接口控制器", tags = "接口控制器")
@RestController
@RequestMapping("/ums/resource")
public class ResourceUmsController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private InterfaceResourceService resourceService;
    /**
     * 授权服务
     */
    @Autowired
    private IOAuthService oAuthService;
    /**
     * 角色服务
     */
    @Autowired
    private IRoleService roleService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @return 所有数据
     */
//    @ApiOperation(value = "条件分页查询Resource")
//    @PostMapping("/searchByCondition")
//    public ResultInfo selectAll(@ApiParam(value = "Page对象", required = true) @RequestBody PageExt<InterfaceResource> page) {
//
//        Page pageData = this.resourceService.selectResourcePage(page);
//
////        pageData.setTotal();
//
//        return ResultUtil.createSuccess(200,pageData);
//    }
    /**
     * 分页查询所有数据(重写)
     */
    @ApiOperation(value = "条件分页查询Resource")
    @PostMapping("/searchByCondition")
    public ResultInfo searchByCondition(@ApiParam(value = "FiltersData对象", required = true)  @RequestBody FiltersData filtersData) {
        int start = (filtersData.getPageIndex() - 1) * filtersData.getPageSize();
        int pageSize = filtersData.getPageSize();
        String filterSql = "where 1=1 ";
        if(!CollectionUtils.isEmpty(filtersData.getFilters())){
            for (FiltersUtils filter : filtersData.getFilters()) {
                if ("menuName".equals(filter.getField())) {
                    filter.setValue(filter.getValue().replace("'", ""));
                }
            }
            filterSql = filtersData.getFilterString();
        }
        String filterSortSql = filterSql;
        if(!StringUtils.isEmpty(filtersData.getSortField()) && filtersData.getSortDirection()!=null){
            filterSortSql += filtersData.getSortString();
        }else{
            filterSortSql += " order by createTime desc ";
        }
        List<InterfaceResource> interfaceResourceList = resourceService.selectResourcePage(filterSortSql,start,pageSize);
        int count = 0;
        if (!CollectionUtils.isEmpty(interfaceResourceList)) {
            count = resourceService.selectCountByCondition(filterSql);
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "操作成功", "查询资源列表成功", interfaceResourceList, count);
    }


    /**
     * 分页查询所有数据
     *
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有Resource")
    @GetMapping("/all")
    public ResultInfo selectAll() {
            return ResultUtil.createSuccess(200, this.resourceService.selectAll());
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "根据id查询 Resource")
    @GetMapping("/{id}")
    public ResultInfo selectOne(@ApiParam(value = "ID", required = true) @PathVariable("id") Integer id) {
        return ResultUtil.createSuccess(200, this.resourceService.selectById(id));
    }

    /**
     * 新增数据
     *
     * @param resource 实体对象
     * @return 新增结果
     */
    @PostMapping("/add")
    @ApiOperation(value = "新增Resource")
    public ResultInfo insert(@ApiParam(value = "Resource", required = true) @RequestBody InterfaceResource resource) {
        String roleidstr = resource.getRoleIds();
        if(roleidstr.endsWith(","))
            roleidstr = roleidstr.substring(0,roleidstr.length()-1);
        String roleId[] = roleidstr.split(",");

        String userid = getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME));

        resource.setCreateUserId(userid);
        boolean ressuccess = this.resourceService.insertRes(resource);

        int suc = roleService.SaveRolesRelatePermission(resource.getResId(),resource.getId(),roleId,
                userid);

        return ResultUtil.createSuccess(200,suc );
    }

    /**
     * 修改数据
     *
     * @param resource 实体对象
     * @return 修改结果
     */
    @PutMapping("/update")
    @ApiOperation(value = "根据id修改Resource")
    public ResultInfo update(@ApiParam(value = "Resource", required = true) @RequestBody InterfaceResource resource) {
        String roleidstr = resource.getRoleIds();

        if(roleidstr == null){
            ResultUtil.createFail(505, null);
        }

        String roleId[] = null;
        if(roleidstr != null ){
            if(roleidstr.endsWith(",")) {
                roleidstr = roleidstr.substring(0, roleidstr.length() - 1);
            }
            roleId = roleidstr.split(",");
        }

        String userid = getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME));
        resource.setCreateUserId(userid);

        if(roleId != null)
            roleService.SaveRolesRelatePermission(resource.getResId(),resource.getId(),roleId,
                    userid);

        return ResultUtil.createSuccess(200, this.resourceService.updateById(resource));
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @DeleteMapping("/delete")
    @ApiOperation(value = "删除Resource")
    public ResultInfo delete(@ApiParam(value = "Resource", required = true) @RequestParam("idList") List<Long> idList) {
        return ResultUtil.createSuccess(200, this.resourceService.deleteBatchIds(idList));
    }

    @GetMapping("menusTree")
    @ApiOperation(value = "获取资源树")
    public ResultInfo menusTree(String type)
    {
//        String roleIdstr = getStaff("roleid");

        return ResultUtil.createSuccess(200,  null);
    }

    @GetMapping("/authType")
    public ResultInfo AuthType()
    {
        List<Option> options = new ArrayList<Option>();
        Option option = new Option( "菜单","1",false) ;
        options.add(option);
        Option option1 = new Option( "操作api","2",false) ;
        options.add(option1);
        Option option2 =  new Option( "顶部菜单","3",false);
        options.add(option2);
        Option option3 =  new Option( "按钮","4",false);
        options.add(option3);

        return ResultUtil.createSuccess(200,  options);
    }

    @GetMapping("/parents")
    public ResultInfo parents(@RequestHeader(Config.JWT_CUSTOMER_TOKEN_NAME) String token) throws OAuthSystemException {
        String authuser = oAuthService.getUsernameByAccessToken(token);

        String[] paramarr = authuser == null?null:authuser.split("_");
        if(authuser == null || paramarr == null || paramarr.length < 7 ){
            throw new OAuthSystemException("prameter : 7 strings concat with _ between");
        }
        String username = paramarr[0];
        String password = paramarr[1];
        String applicationKey = paramarr[2];
        String roleKey = paramarr[3];
        String deptCode = paramarr[4];
        String orgKey = paramarr[5];

        List<Option> options = new ArrayList<Option>();
        roleKey = roleKey.substring(roleKey.length()-1).equals(",")?roleKey.substring(0,roleKey.length()-1):roleKey;
        roleKey = "('"+roleKey.replace(",","','")+"')";
        List<InterfaceResource> resources = this.resourceService.getResourcesByRoleKey("1','2",roleKey,applicationKey,true);
        for(InterfaceResource res : resources)
        {
            Option option = new Option( res.getMenuName(), res.getResId().toString(), res.getStatus() == 1?false:true );
            options.add(option);
        }
        Option option = new Option( "顶级", "0",false );
        options.add(option);
        return ResultUtil.createSuccess(200,  options);
    }


    @GetMapping("/roles")
    public ResultInfo roles(@RequestHeader(Config.JWT_CUSTOMER_TOKEN_NAME) String token) throws OAuthSystemException {

        List<Option> options = new ArrayList<Option>();
        List<Role> roles = this.roleService.selectList(new EntityWrapper<Role>());
        for(Role role : roles)
        {
            if(role!=null){
                Option option = new Option( role.getRoleName(),role.getId(), (role.getRecordState()!=null && role.getRecordState() == 2)?false:true );
                options.add(option);
            }
        }
        return ResultUtil.createSuccess(200,  options);
    }

    @PostMapping("/query")
    @ApiOperation("查询接口信息")
    public ResultInfo query(@ApiParam(value = "Page对象", required = true) @RequestBody PageExt<InterfaceResource> page) {

            Page pageData = this.resourceService.selectResourceByPage(page);

            System.out.println(pageData);

            return ResultUtil.createSuccess(200,pageData);
    }

    /**
     * 新增数据
     *
     * @param resource 实体对象
     * @return 新增结果
     */
    @PostMapping("/addResource")
    @ApiOperation(value = "新增Resource")
    public ResultInfo addResource(@ApiParam(value = "Resource", required = true) @RequestBody InterfaceResource resource) {
        if(resource.getAppId() == null || resource.getAppId() == 0)
            resource.setAppId(51);
        this.resourceService.addResource(resource);
        return ResultUtil.createSuccess(200,null);
    }



}