package com.liangjian11.ums.controller.ums;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.liangjian11.ums.controller.base.BaseController;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.InterfaceResource;
import com.liangjian11.ums.model.Role;
import com.liangjian11.ums.model.common.*;
import com.liangjian11.ums.service.IRoleService;
import com.liangjian11.ums.service.IUserService;
import com.liangjian11.ums.service.InterfaceResourceService;
import com.liangjian11.ums.utils.TreeNode;
import com.liangjian11.ums.utils.content.Config;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;


@Api(value = "角色控制器", tags = "角色控制器")
@RestController
@RequestMapping("/ums/role")
public class RoleController extends BaseController {
    /**
     * 角色服务
     */
    @Autowired
    private IRoleService roleService;
    /**
     * 权限服务
     */
    @Autowired
    private InterfaceResourceService resourceService;
    /**
     * 账户服务
     */
    @Autowired
    private IUserService userService;

    @ApiOperation(value = "条件分页查询Role")
    @PostMapping("/queryRoles")
    public ResultInfo queryRoles(@ApiParam(value = "FiltersData对象", required = true) @RequestBody FiltersData filtersData) {
        if (filtersData.getPageIndex() <= 0 && filtersData.getPageSize() <= 0) {
            return ResultUtil.createFail(new IllegalArgumentException("参数违法,PageIndex、PageSize不能为空或<=0"));
        }
        int start = (filtersData.getPageIndex() - 1) * filtersData.getPageSize();
        int pageSize = filtersData.getPageSize();
        String filterSql = "where 1=1 ";
        if (!CollectionUtils.isEmpty(filtersData.getFilters())) {
            for (FiltersUtils filter : filtersData.getFilters()) {
                if ("roleName".equals(filter.getField())) {
                    filter.setValue(filter.getValue().replace("'", ""));
                }
            }
            filterSql = filtersData.getFilterString();
        }
        String filterSortSql = filterSql;
        if (!StringUtils.isEmpty(filtersData.getSortField()) && filtersData.getSortDirection() != null) {
            filterSortSql += filtersData.getSortString();
        } else {
            filterSortSql += " order by CreationDate desc ";
        }
        List<Role> roleList = roleService.getRolesByCondition(filterSortSql, start, pageSize);
        int count = 0;
        if (!CollectionUtils.isEmpty(roleList)) {
            count = roleService.getCountByCondition(filterSql);
        }
        for (Role role : roleList) {
            if (!StringUtils.isEmpty(role.getLastModifiedUser())) {
                Account account = userService.selectOne(new EntityWrapper<Account>().eq("CAST(Id as varchar(36))", role.getLastModifiedUser()));
                role.setLastModifiedUser(Objects.nonNull(account) ? account.getNickName() : "");
            }
        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "操作成功", "查询所有角色成功", roleList, count);
    }

    @ApiOperation(value = "根据RoleID查询Role及其权限")
    @GetMapping("/queryRoleByRoleId/{roleId}")
    public ResultInfo queryRoleByRoleId(@ApiParam(value = "角色Id", required = true) @PathVariable("roleId") String roleId) {
        return roleService.getRoleByRoleId(roleId);
    }


    @ApiOperation(value = "添加/更新Role")
    @PostMapping("/addOrEditRole")
    public ResultInfo addOrEditRole(@ApiParam(value = "Role对象", required = true) @RequestBody Role role) {
        if (StringUtils.isEmpty(role.getRoleName()) || role.getRoleName().length() > 10) {
            return ResultUtil.createFail(new IllegalArgumentException("创建角色失败,角色名称过长"));
        }
        Date date = new Date();
        List<InterfaceResource> resourceList = role.getResourceList();
        List<InterfaceResource> tempList = new ArrayList<>(resourceList);
        if (!CollectionUtils.isEmpty(resourceList)) {
            outter:for (InterfaceResource interfaceResource : resourceList) {
                InterfaceResource resource = resourceService.selectOne(new EntityWrapper<InterfaceResource>().eq("Id", interfaceResource.getId()));
                InterfaceResource parentRes = resourceService.selectOne(new EntityWrapper<InterfaceResource>().eq("resId", resource.getParentResId()));
                if (Objects.nonNull(parentRes)) {
                    for (InterfaceResource tempResource : tempList) {
                        if (tempResource.getId().equals(parentRes.getId())) {
                            continue outter;
                        }
                    }
                    tempList.add(parentRes);
                }
            }
            role.setResourceList(tempList);
        }
        if (StringUtils.isEmpty(role.getId())) {
            if (roleService.selectCount(new EntityWrapper<Role>().eq("RoleName", role.getRoleName())) > 0) {
                return ResultUtil.createFail(new IllegalArgumentException("创建角色失败,角色名称已存在"));
            }
            role.setId(UUID.randomUUID().toString());
            role.setAppId(50);
            role.setRecordState(2);
            role.setStatus(1);
            role.setCreationDate(date);
            role.setLastModified(date);
            role.setCreateByUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
            role.setLastModifiedUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
            role.setRoleKey(role.getRoleName());
            return roleService.insertRoleWithResources(role);
        } else {
            Role oldRole = roleService.selectOne(new EntityWrapper<Role>().eq("id", role.getId()));
            if (oldRole == null) {
                return ResultUtil.createFail(new IllegalArgumentException("更新角色失败,角色不存在"));
            }
            if (!oldRole.getRoleName().equals(role.getRoleName())) {
                if (roleService.selectCount(new EntityWrapper<Role>().eq("RoleName", role.getRoleName())) > 0) {
                    return ResultUtil.createFail(new IllegalArgumentException("更新角色失败,角色名称已存在"));
                }
            }
            role.setRoleKey(role.getRoleName());
            role.setAppId(50);
            role.setRecordState(2);
            role.setStatus(1);
            role.setLastModified(date);
            role.setLastModifiedUser(getTokenUserId(getTokenUser(Config.JWT_CLIENT_USERNAME)));
        }
        return roleService.updateRoleWithResources(role);
    }


    @ApiOperation(value = "删除Role")
    @PostMapping("/deleteRole")
    public ResultInfo deleteRole(@ApiParam(value = "Page对象", required = true) @RequestBody Role role) {
        if (!StringUtils.isEmpty(role.getId())) {
            return roleService.deleteRoleWithUserAndResource(role.getId());
        } else {
            return ResultUtil.createFail(new IllegalArgumentException("删除角色失败，ID为空"));
        }
    }


    @ApiOperation(value = "树状权限列表")
    @PostMapping("/menuTree")
    public ResultInfo menuTree() {
        List<TreeNode> menuTree = resourceService.listResourceTree();
        int count = 0;
        if (!CollectionUtils.isEmpty(menuTree)) {

        }
        return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS, 200, "操作成功", "查询树状资源成功", menuTree, count);
    }

}

