package com.liangjian11.ums.dao.foo;


import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.liangjian11.ums.model.InterfaceResource;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 权限表(Resource)表数据库访问层
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Repository
public interface InterfaceResourceMapper extends BaseMapper<InterfaceResource> {
    /**
     * 根据用户及项目key查询当前用户拥有的资源
     *
     * @param filters  过滤条件
     * @param pageSql 分页sql
     * @return
     */
    List<InterfaceResource> getResourcesPage(@Param("filters")String filters,@Param("pageSql")String pageSql);


    /**
     * 查询权限列表、分页、过滤、排序
     * @param filterSql
     * @param start
     * @param pageSize
     * @return
     */
    List<InterfaceResource> selectResourcesPage(@Param("filterSql")String filterSql,@Param("start") int start,@Param("pageSize") int pageSize);


    /**
     * 分页查询数目
     * @return
     */
    Integer selectCountByCondition(@Param("filterSql")String filterSql);


    /**
     * 根据用户及项目key查询当前用户拥有的资源
     *
     * @param userId         用户id
     * @param applicationKey 项目key
     * @param andTypes 资源类型
     * @return
     */
    List<InterfaceResource> getResourcesByUserAndAppKey(@Param("userId") String userId, @Param("applicationKey") String applicationKey,@Param("andTypes")String andTypes);

    /**
     * 根据用户及项目key查询当前用户拥有的资源
     *
     * @param clientId         用户客户端id
     * @param applicationKey 项目key
     * @param andTypes 资源类型
     * @return
     */
    List<InterfaceResource> getResourcesByClientAndAppKey(@Param("clientId") String clientId, @Param("applicationKey") String applicationKey,@Param("andTypes")String andTypes);

    /**
     * 获取当前项目所有资源
     *
     * @param applicationKey
     * @return
     */
    List<InterfaceResource> getResourcesByAppKey(@Param("applicationKey") String applicationKey,@Param("andTypes")String andTypes);

    /**
     * 获取当前项目当前角色所有资源
     *
     * @param applicationKey
     * @return
     */
    List<InterfaceResource> getResourcesByRoleKey(@Param("andTypes")String andTypes,@Param("roleKey")String roleKey,@Param("applicationKey") String applicationKey);

    int selectResourceCount(@Param("pageSql") String pageSql);

    List<InterfaceResource> selectResourceByPage(@Param("pageSql") String pageSql);

    void addResource(InterfaceResource resource);

    Integer insertRes(InterfaceResource resource);

    List<InterfaceResource> selectResourceTree();

    List<InterfaceResource> selectResourceByRoleId(@Param("roleId") String roleId);

    InterfaceResource selectById(@Param("Id") String Id);
}