package com.liangjian11.ums.model.common;


import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

@Data
@NoArgsConstructor
public class FiltersData {
    /**
     * 当前页
     */
    private int pageIndex;
    /**
     * 每页数量
     */
    private int pageSize;
    /**
     * 过滤条件
     */
    private List<FiltersUtils> filters;
    /**
     * 升序或降序
     */
    private Integer sortDirection;
    /**
     * 排序字段
     */
    private String sortField;

    public int getPageIndex() {
        return pageIndex;
    }

    public void setPageIndex(int pageIndex) {
        this.pageIndex = pageIndex;
    }

    public int getPageSize() {
        return pageSize;
    }

    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    public List<FiltersUtils> getFilters() {
        return filters;
    }

    public void setFilters(List<FiltersUtils> filters) {
        this.filters = filters;
    }

    public Integer getSortDirection() {
        return sortDirection;
    }

    public void setSortDirection(Integer sortDirection) {
        this.sortDirection = sortDirection;
    }

    public String getSortField() {
        return sortField;
    }

    public void setSortField(String sortField) {
        this.sortField = sortField;
    }

    public  String getFilterString(Boolean lowercase) {
        String queryCondition = " where 1=1 ";
        if (this.getFilters() != null) {
            for (FiltersUtils item : this.getFilters()) {
                if (item != null && (item.getValue()!=null || (item.getRules() != null && item.getRules().size() > 0)) && item.getValue()!=null&& !item.getValue().trim().equals("")) {
                    String val = item.getValue();
                    String field = item.getField();

                    if(lowercase !=null && lowercase.equals(true))
                        field = field.toLowerCase();

                    String operate = item.getOperate();
                    switch (operate) {
                        case "equal":
                            queryCondition += " and " + field + "=  '" + val + "'";
                            break;
                        case "notequal":
                            queryCondition += " and " + field + "!=  '" + val+"'";

                            break;
                        case "less":
                            queryCondition += " and " + field + "< '" + val+"'";

                            break;
                        case "lessorequal":
                            queryCondition += " and " + field + "<= '" + val+"'";

                            break;
                        case "greater":
                            queryCondition += " and " + field + "> '" + val+"'";

                            break;
                        case "greaterorequal":
                            queryCondition += " and " + field + ">= '" + val+"'";

                            break;
                        case "contains":
                            queryCondition += " and " + field + " like '%" + val + "%'";

                            break;
                        case "startswith":
                            queryCondition += " and " + field + " >='" + val + "'";

                            break;
                        case "endswith":
                            queryCondition += " and " + field + " <= '" + val + "'";

                            break;
                        case "or":
                            String innerCondition = "1 = 1";
                            List<Map<String, Object>> rulesMap = item.getRules();
                            for (Map<String, Object> stringObjectMap : rulesMap) {
                                String rulesField = (String) stringObjectMap.get("field");
                                String rulesValue = (String) stringObjectMap.get("value");
                                innerCondition += " or "+rulesField+" = '"+rulesValue+"'";
                            }

                            innerCondition = innerCondition.replace("1 = 1 or", "");
                            queryCondition += " and (" + innerCondition + ")";

                            break;
                        case "nestand":
                            String innerConditionNest = "1 = 1";
                            List<Map<String, Object>> rules = item.getRules();
                            for (Map<String, Object> stringObjectMap : rules) {
                                String rulesField = (String) stringObjectMap.get("field");
                                String rulesvalue = (String) stringObjectMap.get("value");
                                innerConditionNest += " and "+rulesField+" = '"+rulesvalue+"'";
                            }
                            innerConditionNest = innerConditionNest.replace("1 = 1 and", "");
                            queryCondition += " and (" + innerConditionNest + ")";

                            break;
                    }
                }
            }
        }


        return queryCondition;
    }


    public String getSortString(){
        String orderCondition = "";
        if (this.getSortDirection() != null && !StringUtils.isEmpty(this.getSortField())) {
            //this.setSortField(FiltersUtils.sortBigToLine(this.getSortField()));
            if (this.getSortDirection() == 1) {
                orderCondition += "order by " + this.getSortField() + " desc";
            } else if (this.getSortDirection() == 0) {
                orderCondition += "order by " + this.getSortField() + " asc";
            }
        }
        return orderCondition;
    }

    public  String getFilterString() {
        return this.getFilterString(true);
    }
//
//    public static HashMap<String,String> getFieldTypes(Class clasz){
//        HashMap<String,String> hashMap = new HashMap<String,String>(10);
//        java.lang.reflect.Field[] flds = clasz.getClass().getDeclaredFields();  //遍历所有属性
//        if ( flds != null )
//        {
//            for ( int i = 0; i < flds.length; i++ )
//            {
//                String fldType = flds[i].getGenericType().toString();
//                //System.out.println(fldType);
//
//                String fldName = flds[i].getName(); //得到属性值
//                //System.out.println(fldName+":"+o);  //得到属性类型
//                hashMap.put(fldName,fldType);
//            }
//        }
//    }
//
//
//    public  String getFilterParam(List<SqlParameterValue> valuesList,Class clasz) {
//        String queryCondition = " where 1=1 ";
//        HashMap<String,String> fieldTypes = this.getFieldTypes(clasz);
//        HashMap<String,SqlParameter> paramsList = null;
//        if(paramsList == null && clasz!=null) {
//            paramsList = new HashMap<String, SqlParameter>(this.getFilters().size());
//        }
//
//        if(valuesList == null) {
//            valuesList = new ArrayList<>(this.getFilters().size());
//        }
//
//        if (this.getFilters() != null) {
//            int i = 0;
//            for (FiltersUtils item : this.getFilters()) {
//                if (item != null && (item.getValue()!=null || (item.getRules() != null && item.getRules().size() > 0)) && item.getValue()!=null&& !item.getValue().trim().equals("")) {
//                    String val = item.getValue();
//                    String field = item.getField();
//                    String operate = item.getOperate();
//                    if(field.indexOf("time")>0 && operate.equals("equal")){
//                        operate="contains";
//                    }
//                    switch (operate) {
//                        case "equal":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                if(fieldTypes.containsKey(field))
//                                    type =
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field +"=  @" + field + "_"+i + " ";
//                            break;
//                        case "notequal":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + "!=  @" + field + "_"+i+"'";
//
//                            break;
//                        case "less":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + "< @" + field + "_"+i+"'";
//
//                            break;
//                        case "lessorequal":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + "<= @" + field + "_"+i+"'";
//
//                            break;
//                        case "greater":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + "> @" + field + "_"+i+"'";
//
//                            break;
//                        case "greaterorequal":
//                            if(!paramsList.containsKey(field + "_"+i)){
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_"+i,new SqlParameter(field + "_"+i,type));
//                            }
//                            queryCondition += " and " + field + ">= @" + field + "_"+i+"'";
//
//                            break;
//                        case "contains":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + " like @%" + field + "_"+i + "% ";
//
//                            break;
//                        case "startswith":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + " >= @" + field + "_"+i + " ";
//
//                            break;
//                        case "endswith":
//                            if(!paramsList.containsKey(field + "_"+i)) {
//                                int type = Types.VARCHAR;
//                                paramsList.put(field + "_" + i, new SqlParameter(field + "_" + i, type));
//                            }
//                            queryCondition += " and " + field + " <= @" + field + "_"+i + " ";
//
//                            break;
//                        case "or":
//                            String innerCondition = "1 = 1";
//                            List<Map<String, Object>> rulesMap = item.getRules();
//                            int j = 0;
//                            for (Map<String, Object> stringObjectMap : rulesMap) {
//                                String rulesField = (String) stringObjectMap.get("field");
//                                String rulesValue = (String) stringObjectMap.get("value");
//
//                                if(!paramsList.containsKey(rulesField + "_"+i+"_or_"+j)) {
//                                    int type = Types.VARCHAR;
//                                    paramsList.put(rulesField + "_" + i+"_or_"+j, new SqlParameter(rulesField + "_" + i+"_or_"+j, type));
//                                }
//                                innerCondition += " or "+rulesField+" = @"+rulesValue+"" + "_" + i+"_or_"+j;
//
//                                j++;
//                            }
//
//                            innerCondition = innerCondition.replace("1 = 1 or", "");
//                            queryCondition += " and (" + innerCondition + ")";
//
//                            break;
//                        case "nestand":
//                            String innerConditionNest = "1 = 1";
//                            List<Map<String, Object>> rules = item.getRules();
//                            for (Map<String, Object> stringObjectMap : rules) {
//                                String rulesField = (String) stringObjectMap.get("field");
//                                String rulesvalue = (String) stringObjectMap.get("value");
//                                innerConditionNest += " and "+rulesField+" = '"+rulesvalue+"'";
//                            }
//                            innerConditionNest = innerConditionNest.replace("1 = 1 and", "");
//                            queryCondition += " and (" + innerConditionNest + ")";
//
//                            break;
//                    }
//                }
//                i++;
//            }
//        }
//        return queryCondition;
//    }
}
