package com.liangjian11.ums.service;

import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.Role;
import com.baomidou.mybatisplus.service.IService;
import com.liangjian11.ums.web.result.ResultInfo;

import java.util.List;

public interface IRoleService extends IService<Role> {

    /**
     * 根据条件查询角色列表(分页)
     * @return
     */
    List<Role> getRolesByCondition(String filterSql,Integer start,Integer pageSize);

    /**
     * 根据条件查询出来的行数
     * @param filterSql
     * @return
     */
    Integer getCountByCondition(String filterSql);

    /**
     * 根据RoleID查询Role及其权限
     *
     * @param roleId
     * @return
     */
    ResultInfo getRoleByRoleId(String roleId);

    /**
     *删除角色及关联的用户和权限
     *
     * @param roleId
     * @return
     */
    ResultInfo deleteRoleWithUserAndResource(String roleId);

    /**
     * 新增角色及关联的权限
     *
     * @param role
     * @return
     */
    ResultInfo insertRoleWithResources(Role role);


    /**
     * 更新角色及关联的权限
     *
     * @param role
     * @return
     */
    ResultInfo updateRoleWithResources(Role role);

    /**
     * 查询当前用户拥有的角色集合
     * @param accountId
     * @return
     */
    List<Role> listRoleByUser(String accountId);


    /**
     * 查询当前用户拥有当前系统的角色集合
     *
     * @param currentUser
     * @param applicationKey
     * @return
     */
    List<Role> getRolesByUserAndAppKey(Account currentUser, String applicationKey);

    /**
     * 判断是否拥有某一个角色
     *
     * @param roles
     * @param roleKey
     * @return
     */
    boolean hasRole(List<Role> roles, String roleKey);


    int SaveRolesRelatePermission(int resId,String menuId,String[] rolesIds,String accountId);
}
