package com.liangjian11.ums.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.liangjian11.ums.dao.foo.InterfaceResourceMapper;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.InterfaceResource;
import com.liangjian11.ums.model.common.FilterRule;
import com.liangjian11.ums.model.common.PageExt;
import com.liangjian11.ums.utils.TreeNode;
import com.liangjian11.ums.service.InterfaceResourceService;
import com.liangjian11.ums.utils.content.Config;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 权限表(Resource)表服务实现类
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Service("resourceService")
public class InterfaceResourceServiceImpl extends ServiceImpl<InterfaceResourceMapper, InterfaceResource> implements InterfaceResourceService {
    
    @Autowired
    private InterfaceResourceMapper resourceDao;

    @Override
    public List<InterfaceResource> selectAll() {
        return resourceDao.selectList(null);
    }


    @Override
    public PageExt<InterfaceResource> selectResourcePage(PageExt<InterfaceResource> page) {
        // 不进行 count sql 优化，解决 MP 无法自动优化 SQL 问题
        // page.setOptimizeCountSql(false);
        // 不查询总记录数
        // page.setSearchCount(false);
        // 注意！！ 分页 total 是经过插件自动 回写 到传入 page 对象
        Integer counts = resourceDao.selectCount(new EntityWrapper<InterfaceResource>());
        String pageSql = "order by r.CreateTime desc  " +
                " offset "+(page.gePageIndex()-1)*page.getSize()+" rows fetch next "+page.getSize()+" rows only ";
//        List<InterfaceResource> data= resourceDao.selectPage(page,new EntityWrapper<InterfaceResource>());
        List<InterfaceResource> data= resourceDao.getResourcesPage("",pageSql);
        page.setRecords(data);
        page.setTotal(counts);
        return page;
    }


    @Override
    public  List<InterfaceResource>  selectResourcePage(String filterSql ,int start, int pageSize) {
        return resourceDao.selectResourcesPage(filterSql, start, pageSize);
    }

    @Override
    public Integer selectCountByCondition(String filterSql) {
        return resourceDao.selectCountByCondition(filterSql);
    }

    @Override
    @Transactional(readOnly = true)
    public List<InterfaceResource> getResourcesByUserAndAppKey(Account currentAccount, String applicationKey, Integer types,String grantType) {

        List<InterfaceResource> SysMenuList = null;

        if (Objects.nonNull(currentAccount) &&
                ( Objects.nonNull(currentAccount.getId()) || Objects.nonNull(currentAccount.getLoginId())) &&
                Objects.nonNull(applicationKey)) {
            String andtypes = "";
            if(!types.equals( -1))
                andtypes = "and r.types="+types;

            if(!StringUtils.isEmpty(grantType) &&
             grantType.equals(Config.ClientCredentials))
                SysMenuList = this.baseMapper.getResourcesByClientAndAppKey(currentAccount.getLoginId(), applicationKey,andtypes);
            else
                SysMenuList = this.baseMapper.getResourcesByUserAndAppKey(currentAccount.getId(), applicationKey,andtypes);
        }

        return SysMenuList;
    }

    @Override
    public List<InterfaceResource> getResourcesByAppKey(String applicationKey,Integer types) {

        List<InterfaceResource> SysMenuList = null;

        if (StringUtils.isNotBlank(applicationKey)) {
            String andtypes = "";
            if(!types.equals( -1))
                andtypes = "and r.types="+types;
            SysMenuList = this.baseMapper.getResourcesByAppKey(applicationKey,andtypes);
        }

        return SysMenuList;
    }

    @Override
    public List<InterfaceResource> getResourcesByRoleKey(String authTypes,String roleKey,String applicationKey,Boolean parent) {

        List<InterfaceResource> SysMenuList = null;

        if (StringUtils.isNotBlank(applicationKey)) {
            String  andtypes = "and r.types in ('"+authTypes+"')";
            if(parent!=null && parent.booleanValue() ) andtypes = andtypes + " and r.parentResId = 0 ";
            SysMenuList = this.baseMapper.getResourcesByRoleKey(andtypes,roleKey,applicationKey);
        }

        return SysMenuList;
    }

    @Override
    public Page selectResourceByPage(PageExt<InterfaceResource> page) {
        List<FilterRule> filters = page.getFilters();
        StringBuilder pageSql = new StringBuilder(" where 1 = 1 ");
        filters.forEach(filter -> {
            if (!"".equals(filter.getField()) && !"".equals(filter.getValue())) pageSql.append(filter.toSqlString());
        });
        int count = resourceDao.selectResourceCount(new String(pageSql));

        pageSql.append(" order by CreateTime desc offset ")
                .append((page.gePageIndex() - 1) * page.getSize())
                .append(" rows fetch next ").append(page.getSize()).append(" rows only ");

        List<InterfaceResource> interfaceResourceList = resourceDao.selectResourceByPage(new String(pageSql));
        page.setRecords(interfaceResourceList);
        page.setTotal(count);
        return page;
    }

    @Override
    public void addResource(InterfaceResource resource) {
        resource.setTypes(2);
        resourceDao.addResource(resource);
    }

    public Boolean insertRes(InterfaceResource resource){
        return retBool(resourceDao.insertRes(resource));
    }

    @Override
    public List<TreeNode> listResourceTree() {
        List<InterfaceResource> interfaceResourceList = resourceDao.selectResourceTree();
        List<TreeNode> treeNodeList = new ArrayList<>();
        for (InterfaceResource interfaceResource : interfaceResourceList) {
            TreeNode treeNode = new TreeNode();
            treeNode.setId(interfaceResource.getResId());
            treeNode.setParentId(interfaceResource.getParentResId());
            treeNode.setKey(interfaceResource.getId());
            treeNode.setTitle(interfaceResource.getMenuName());
            treeNodeList.add(treeNode);
        }
        return TreeNode.RecursiveAddress(treeNodeList);
    }

}