package com.liangjian11.ums.service.impl;

import com.liangjian11.ums.service.ClientService;
import com.liangjian11.ums.service.IOAuthService;
//import org.apache.shiro.cache.ehcache.EhCacheManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * OAuth相关处理：authCode 添加/移除  accessToken的添加/移除 。。。。。
 * */
@Service
public class OAuthServiceImpl implements IOAuthService {

    private static RedisTemplate cache;

    @Autowired
    ClientService clientService;

    public static final String AUTHKEY_PRE = "shiro:cache:com.liangjian11.ums.web.shiro.JWTRealm.authorizationCache:";
    public static final String ACSTOKEN_PRE = "shiro:cache:com.liangjian11.ums.web.shiro.JWTRealm.authorizationCache:";

    @Value("${jwt.token.active.time}")
    private static long activeTime = 2 * 60 * 60 * 1000;


    @Autowired
    public OAuthServiceImpl(RedisTemplate redisTemplate) {
        OAuthServiceImpl.cache = redisTemplate;
    }

    @Override
    public void addAuthCode(String authCode, String username) {
        cache.opsForValue().set(AUTHKEY_PRE+authCode, username, 1 * 10, TimeUnit.MILLISECONDS);
    }

    @Override
    public String removeAuthCode(String authCode) {
        return cache.delete(AUTHKEY_PRE+authCode)?authCode:"";
    }

    @Override
    public void addAccessToken(String accessToken, String usernamedata) {//username 是用户数据
        cache.opsForValue().set(ACSTOKEN_PRE+accessToken,usernamedata, activeTime, TimeUnit.MILLISECONDS);
        String[] usernameFirst = usernamedata.split("_");
        if(usernameFirst.length>0) {
            cache.opsForValue().set(ACSTOKEN_PRE + usernameFirst[0], accessToken, activeTime, TimeUnit.MILLISECONDS);
        }
    }

    @Override
    public String getAccessToken(String username) {//username是用户标识
        return (String)cache.opsForValue().get(ACSTOKEN_PRE+username);
    }

    @Override
    public String getAccessUserByToken(String token) {//username是用户标识
        return (String)cache.opsForValue().get(ACSTOKEN_PRE+token);
    }

    @Override
    public String removeAccessToken(String accessToken) {

        return cache.delete(ACSTOKEN_PRE+accessToken)?accessToken:"";
    }

    @Override
    public String removeAccessToken(String accessToken,String usernamedata) {
        Boolean b1 = cache.delete(ACSTOKEN_PRE+accessToken);
        String[] usernameFirst = usernamedata.split("_");
        Boolean b2 = false;
        if(usernameFirst.length>0) {
            b2 = cache.delete(ACSTOKEN_PRE + usernamedata);
        }
        return b1&&b2?accessToken:"";
    }

    @Override
    public boolean checkAuthCode(String authCode) {
        return Objects.nonNull(cache.opsForValue().get(AUTHKEY_PRE+authCode));
    }

    @Override
    public boolean checkAccessToken(String accessToken) {
        return Objects.nonNull(cache.opsForValue().get(ACSTOKEN_PRE+accessToken));
    }

    @Override
    public String getUsernameByAuthCode(String authCode) {
        return (String) cache.opsForValue().get(AUTHKEY_PRE+authCode);
    }

    @Override
    public String getUsernameByAccessToken(String accessToken) {
        return (String) cache.opsForValue().get(ACSTOKEN_PRE+accessToken);
    }

    @Override
    public long getExpireIn() {
        // 此处模拟写成固定值
        return activeTime;
    }

    @Override
    public boolean checkClientId(String clientId) {
        return Objects.nonNull(clientService.findByClientId(clientId));
    }

    @Override
    public boolean checkClientSecret(String clientSecret) {
        return Objects.nonNull(clientService.findByClientSecret(clientSecret));
    }
}
