package com.liangjian11.ums.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.liangjian11.ums.dao.foo.InterfaceResourceMapper;
import com.liangjian11.ums.dao.foo.RoleMapper;
import com.liangjian11.ums.dao.foo.RoleResourceMapper;
import com.liangjian11.ums.dao.foo.UserRoleMapper;
import com.liangjian11.ums.model.*;
import com.liangjian11.ums.service.IRoleService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
@Transactional(transactionManager = "fooManager")
public class RoleServiceImpl extends ServiceImpl<RoleMapper, Role> implements IRoleService {

    @Autowired
    private RoleMapper roleMapper;
    @Autowired
    private RoleResourceMapper roleResourceMapper;
    @Autowired
    private UserRoleMapper userRoleMapper;
    @Autowired
    private InterfaceResourceMapper resourceMapper;

    /**
     * 根据条件查询角色列表(分页)
     * @return
     */
    @Override
    @Transactional(readOnly = true)
    public List<Role> getRolesByCondition(String filterSql, Integer start, Integer pageSize) {
        return roleMapper.selectByCondition(filterSql, start, pageSize);
    }

    /**
     * 根据条件查询出来的行数
     * @param filterSql
     * @return
     */
    @Override
    @Transactional(readOnly = true)
    public Integer getCountByCondition(String filterSql) {
        return roleMapper.selectCountByCondition(filterSql);
    }

    /**
     * 根据RoleID查询Role及其权限
     *
     * @param roleId
     * @return
     */
    @Override
    @Transactional(readOnly = true)
    public ResultInfo getRoleByRoleId(String roleId) {
        Role role = roleMapper.selectRoleByRoleId(roleId);
        if(role!=null){
            return new ResultInfo(ResultInfo.TYPE_RESULT_SUCCESS,200,"操作成功","查询角色详情成功",role,1);
        }
        return ResultUtil.createFail(new IllegalArgumentException("无此角色"));
    }


    @Override
    @Transactional(readOnly = true)
    public List<Role> getRolesByUserAndAppKey(Account currentUser, String applicationKey) {
        List<Role> roleList = Lists.newArrayList();

        if (Objects.nonNull(currentUser) && Objects.nonNull(currentUser.getId())) {
            roleList = this.baseMapper.getRolesByUserAndAppKey(currentUser.getId(), applicationKey);
        }

        return roleList;
    }

    @Override
    @Transactional(readOnly = true)
    public List<Role> listRoleByUser(String accountId) {
        return roleMapper.selectByUser(accountId);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = RuntimeException.class)
    public ResultInfo insertRoleWithResources(Role role) {
        String userId =  role.getLastModifiedUser();
        String roleId =role.getId();
        insert(role);
        int roleResourceDeleteResult = roleResourceMapper.delete(new EntityWrapper<RoleResource>().eq("RoleId",roleId));
        if(!CollectionUtils.isEmpty(role.getResourceList())){
            for (InterfaceResource interfaceResource : role.getResourceList()) {
                String menuId = interfaceResource.getId();
                InterfaceResource tempResource = resourceMapper.selectById(menuId);
                Integer resId = null;
                if(tempResource!=null){
                    resId =tempResource.getResId();
                }else{
                    return ResultUtil.createFail(new IllegalArgumentException("没有此角色"));
                }
                String date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(role.getLastModified());
                Integer status  = 1;
                String sql = "insert into  RightOfRole ( RoleId, MenuId,ResourceId,CreateByUser,CreationTime,LastModify,Status) values ("
                        + (roleId==null ? "null": ("CONVERT(uniqueidentifier,'" + roleId + "')"))+","
                        + (menuId == null ? "null" : ("CONVERT(uniqueidentifier,'" + menuId + "')"))+ ","
                        + (resId == null ? "null" :resId)+ ","
                        + (userId == null ? "null": ("CONVERT(uniqueidentifier,'" + userId + "')"))+","
                        + "'"+date + "','" + date+"'," +status
                        + ") ";
                roleResourceMapper.SaveRolesRelResource(sql);
            }
            return ResultUtil.createSuccess(200, "创建账户成功");
        }
        return ResultUtil.createSuccess(200, "创建账户成功");
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = RuntimeException.class)
    public ResultInfo updateRoleWithResources(Role role) {
        String userId =  role.getLastModifiedUser();
        String roleId = role.getId();
        boolean insertRoleResult = updateById(role);
        int roleResourceDeleteResult = roleResourceMapper.delete(new EntityWrapper<RoleResource>().eq("RoleId",roleId));
        if(!CollectionUtils.isEmpty(role.getResourceList())){
            for (InterfaceResource interfaceResource : role.getResourceList()) {
                String menuId = interfaceResource.getId();
                InterfaceResource tempResource = resourceMapper.selectById(menuId);
                Integer resId = null;
                if(tempResource!=null){
                    resId =tempResource.getResId();
                }else{
                    return ResultUtil.createFail(new IllegalArgumentException("没有此角色"));
                }
                String date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(role.getLastModified());
                Integer status  = 1;
                String sql = "insert into  RightOfRole (RoleId, MenuId,ResourceId,CreateByUser,CreationTime,LastModify,Status) values ("
                        + (roleId==null ? "null": ("CONVERT(uniqueidentifier,'" + roleId + "')"))+","
                        + (menuId == null ? "null" : ("CONVERT(uniqueidentifier,'" + menuId + "')"))+ ","
                        + (resId == null ? "null" :resId)+ ","
                        + (userId == null ? "null": ("CONVERT(uniqueidentifier,'" + userId + "')"))+","
                        + "'"+date + "','" + date+"'," +status
                        + ") ";
                roleResourceMapper.SaveRolesRelResource(sql);
            }
            return ResultUtil.createSuccess(200, "更新账户成功");
        }
        return ResultUtil.createSuccess(200, "更新账户成功");
    }




    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = RuntimeException.class)
    public ResultInfo deleteRoleWithUserAndResource(String roleId) {
        boolean result = delete(new EntityWrapper<Role>().eq("Id", roleId));
        System.out.println("删除role结果:" + result);

        int i1 = userRoleMapper.delete(new EntityWrapper<UserRole>().eq("RoleId", roleId));
        System.out.println("删除UserRole结果:" + i1);

        int i2 = roleResourceMapper.delete(new EntityWrapper<RoleResource>().eq("RoleId", roleId));
        System.out.println("删除RoleResource结果:" + i2);
        return ResultUtil.createSuccess(200, "删除角色成功");
    }

    /**
     * 判断是否超级管理员,条件:是否拥有roleKey为admin的角色
     *
     * @param roles
     * @return
     */
    @Override
    @Transactional(readOnly = true)
    public boolean hasRole(List<Role> roles, String roleKey) {

        if (StringUtils.isNotBlank(roleKey) && Objects.nonNull(roles) && roles.size() > 0) {
            for (Role role : roles) {
                if (roleKey.equalsIgnoreCase(role.getRoleKey())) {
                    return true;
                }
            }

        }
        return false;
    }


    @Override
    public int SaveRolesRelatePermission(int resId, String menuId, String[] rolesIds, String accountId) {
        String sql = "update RightOfRole set status =0 where ResourceId='" + resId + "';";
        int index = 0;
        for (String item : rolesIds) {
            index++;
            sql += " if (exists(select * from RightOfRole where ResourceId =  '" + resId + "' and CAST(RoleId as varchar(36)) = '" + item + "' and Status = 0)) " +
                    " begin " +
                    " update RightOfRole set Status = 1 , UpdateByUser = '" + accountId + "',menuId = " + (menuId == null ? "null" : "'" + menuId + "'") + "   where ResourceId = '" + resId + "' " +
                    "and CAST(RoleId as varchar(36)) =  '" + item + "' and Status = 0 " +
                    " end " +
                    " else " +
                    " begin " +
                    " insert into RightOfRole (RoleId, MenuId,ResourceId,CreateByUser) " +
                    " values (CONVERT(uniqueidentifier,'" + item + "')," + (menuId == null ? "null" : ("CONVERT(uniqueidentifier,'" + menuId + "')")) + ",'" + resId + "',CONVERT(uniqueidentifier,'" + accountId + "')) end ";
        }
        this.roleResourceMapper.SaveRolesRelResource(sql);

        return 1;
    }
}
