package com.liangjian11.ums.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.liangjian11.ums.dao.foo.AccountMapper;
import com.liangjian11.ums.dao.foo.RoleMapper;
import com.liangjian11.ums.dao.foo.UserMapper;
import com.liangjian11.ums.dao.foo.UserRoleMapper;
import com.liangjian11.ums.model.Account;
import com.liangjian11.ums.model.Role;
import com.liangjian11.ums.model.UserRole;
import com.liangjian11.ums.service.IUserService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.liangjian11.ums.web.result.ResultInfo;
import com.liangjian11.ums.web.result.ResultUtil;
import org.apache.catalina.User;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;
import java.util.UUID;

@Service("iUserService")
@Transactional(transactionManager = "fooManager")
public class UserServiceImpl extends ServiceImpl<AccountMapper, Account> implements IUserService {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private RoleMapper roleMapper;

    @Override
    @Transactional(readOnly = true)
    public List<Account> getUserByCondition(String filterSql, Integer start, Integer pageSize) {
        return userMapper.selectByCondition(filterSql, start, pageSize);
    }

    @Override
    @Transactional(readOnly = true)
    public Integer selectCountByCondition(String filterSql) {
        return userMapper.selectCountByCondition(filterSql);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = RuntimeException.class)
    public ResultInfo insertUserWithRole(Account account) {
        this.insert(account);
        if (!CollectionUtils.isEmpty(account.getRoleList())) {
            userRoleMapper.delete(new EntityWrapper<UserRole>().eq("AccountId", account.getId()));
            for (Role role : account.getRoleList()) {
                UserRole userRole = new UserRole();
                if (roleMapper.selectCount(new EntityWrapper<Role>().eq("id", role.getId())) > 0) {
                    userRole.setId(UUID.randomUUID().toString());
                    userRole.setAccountId(account.getId());
                    userRole.setRoleId(role.getId());
                    userRole.setStatus(1);
                    userRole.setCreateByUser(account.getCreateByUser());
                    userRole.setLastModifiedUser(account.getCreateByUser());
                    userRole.setCreationTime(account.getLastModified());
                    userRole.setLastModified(account.getLastModified());
                } else {
                    return ResultUtil.createFail(new IllegalArgumentException("创建角色失败,无此角色"));
                }
                userRoleMapper.insert(userRole);
            }
        }
        return ResultUtil.createSuccess(200, "创建角色成功");
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = RuntimeException.class)
    public ResultInfo updateUserWithRole(Account account) {
        this.updateById(account);
        if (!CollectionUtils.isEmpty(account.getRoleList())) {
            userRoleMapper.delete(new EntityWrapper<UserRole>().eq("AccountId", account.getId()));
            for (Role role : account.getRoleList()) {
                UserRole userRole = new UserRole();
                if (roleMapper.selectCount(new EntityWrapper<Role>().eq("id", role.getId())) > 0) {
                    userRole.setId(UUID.randomUUID().toString());
                    userRole.setAccountId(account.getId());
                    userRole.setRoleId(role.getId());
                    userRole.setStatus(1);
                    userRole.setCreateByUser(account.getUpdateByUser());
                    userRole.setLastModifiedUser(account.getUpdateByUser());
                    userRole.setCreationTime(account.getLastModified());
                    userRole.setLastModified(account.getLastModified());
                } else {
                    return ResultUtil.createFail(new IllegalArgumentException("更新角色失败,无此角色"));
                }
                userRoleMapper.insert(userRole);
            }
        }
        return ResultUtil.createSuccess(200, "更新角色成功");
    }



    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = RuntimeException.class)
    public ResultInfo deleteUserWithRole(String userId) {
        EntityWrapper<Account> userEntityWrapper = new EntityWrapper<>();
        userEntityWrapper.eq("id", userId);
        if (this.delete(userEntityWrapper)) {
            userRoleMapper.delete(new EntityWrapper<UserRole>().eq("AccountId", userId));
            return ResultUtil.createSuccess(200, "删除角色成功");
        }
        return ResultUtil.createFail(new RuntimeException("删除角色失败"));
    }

    @Override
    @Transactional(readOnly = true)
    public String selectMaxAccountCode() {
        return userMapper.selectMaxAccountCode();
    }


    @Override
    @Transactional(readOnly = true)
    public Account getUserByUserName(String username) {
        if (StringUtils.isBlank(username)) {
//            throw new Exception("Params invalid!", 504);
            throw new AuthenticationException("Params invalid!" + 504);
        }
//        EntityWrapper entityWrapper = new EntityWrapper<Account>();
//        entityWrapper.eq("loginid", username);
//        entityWrapper.eq("RecordState", 2);
        return this.baseMapper.selectByUserName(username, 2);
    }

    //    @Override
    public Account loadUserByUsername(String userName) throws AuthenticationException {

        Account user = this.getUserByUserName(userName);
        if (Objects.isNull(user)) {
            throw new AuthenticationException("User doesn't existed!");
        }
        return user;
    }
}
