package com.liangjian11.ums.service.impl.organization;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.liangjian11.ums.dao.foo.ClientMapper;
import com.liangjian11.ums.dao.foo.organization.OrgdocMapper;
import com.liangjian11.ums.model.Client;
import com.liangjian11.ums.model.InterfaceResource;
import com.liangjian11.ums.model.common.FilterRule;
import com.liangjian11.ums.model.common.PageExt;
import com.liangjian11.ums.model.organization.Orgdoc;
import com.liangjian11.ums.service.organization.OrgdocService;
import com.liangjian11.ums.utils.CreateSecrteKey;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static com.liangjian11.ums.utils.CreateSecrteKey.getPublicKey;

/**
 * (Orgdoc)表服务实现类
 *
 * @author hsw
 * @since 2020-04-14 14:12:53
 */
@Service("orgdocService")
public class OrgdocServiceImpl extends ServiceImpl<OrgdocMapper, Orgdoc> implements OrgdocService {

    @Autowired
    private OrgdocMapper orgdocDao;

    @Autowired
    private ClientMapper clientMapper;


    /**
     * 新增一个机构的同时创建一个客户端
     *
     * @param orgdoc
     */
    @Override
    public void add(Orgdoc orgdoc) throws Exception {
        String randomId = new SimpleDateFormat("MMddhhmmss").format(new Date());
        orgdoc.setOrgId(Integer.valueOf("0" + randomId));
        orgdoc.setSecret(UUID.randomUUID().toString().replace("-", ""));
        orgdoc.setOrgKey(UUID.randomUUID().toString().replace("-", ""));

        Map<String, Object> keyMap = CreateSecrteKey.initKey();
        String publicKey = CreateSecrteKey.getPublicKey(keyMap);
        String privateKey = CreateSecrteKey.getPrivateKey(keyMap);

        // 创建一个客户端
        Client client = new Client();
        client.setClientSecret(publicKey);
        client.setClientPrivSecret(privateKey);
        client.setClientSecretType("RSA");
        client.setClientName(orgdoc.getOrgName());
        clientMapper.createClient(client);
        Client client1 = clientMapper.selectByClientId(client.getClientId());

        // 将新创建的客户端的id赋值给机构
        orgdoc.setOauth2ClientId(client1.getId());


        orgdoc.setClientId(client1.getClientId());
        orgdoc.setClientSecret(client1.getClientSecret());
        orgdoc.setClientPrivSecret(client1.getClientPrivSecret());
        orgdoc.setClientSecretType(client1.getClientSecretType());

        this.orgdocDao.insert(orgdoc);
    }

    /**
     * 新增一个机构的同时创建一个客户端
     *
     * @param orgdoc
     */
    @Override
    public boolean updateByOrgId(Orgdoc orgdoc) throws Exception {
        Client client1 = clientMapper.selectBySecret(orgdoc.getClientSecret());
        if( client1 != null ) {
            orgdoc.setOauth2ClientId(client1.getId());

            orgdoc.setClientId(client1.getClientId());
            orgdoc.setClientSecret(client1.getClientSecret());
            orgdoc.setClientPrivSecret(client1.getClientPrivSecret());
            orgdoc.setClientSecretType(client1.getClientSecretType());
        }else{
            Map<String, Object> keyMap = CreateSecrteKey.initKey();
            String publicKey = CreateSecrteKey.getPublicKey(keyMap);
            String privateKey = CreateSecrteKey.getPrivateKey(keyMap);
            // 创建一个客户端
            Client client = new Client();
            client.setClientId(UUID.randomUUID().toString());
            client.setClientSecret(publicKey);
            client.setClientPrivSecret(privateKey);
            client.setClientSecretType("RSA");
            client.setClientName(orgdoc.getOrgName());
            clientMapper.createClient(client);

            // 将新创建的客户端的id赋值给机构
            orgdoc.setOauth2ClientId(client.getId());
        }

        return this.retBool(this.orgdocDao.updateById(orgdoc));
    }

    @Override
    public Page selectOrgdocPage(PageExt<InterfaceResource> page) {

        List<FilterRule> filters = page.getFilters();
        StringBuilder pageSql = new StringBuilder(" where 1 = 1 ");
        filters.forEach(filter -> {
            if (!"".equals(filter.getField()) && !"".equals(filter.getValue())) pageSql.append(filter.toSql());
        });
        int count = orgdocDao.selectOrgdocCount(new String(pageSql));

        pageSql.append(" order by CreateTime desc offset ")
                .append((page.gePageIndex() - 1) * page.getSize())
                .append(" rows fetch next ").append(page.getSize()).append(" rows only ");

        List<Orgdoc> orgdocList = orgdocDao.selectOrgdocPage(new String(pageSql));
        page.setRecords(orgdocList);
        page.setTotal(count);
        return page;
    }
}