package com.liangjian11.ums.utils;

import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;

public class RequestCookieHelper {

    public static String getToken(String key){
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
//        HttpServletResponse response = requestAttributes.getResponse();
        return request.getHeader(key);
    }


    /**
     * 获得Cookie的值
     * @param cookieName
     * @return
     */
    public static String GetCookieValue(String cookieName)
    {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        if (request != null)
            for(Cookie cookie : request.getCookies()){
                //如果找到同名cookie，就将value设置为null，将存活时间设置为0，再替换掉原cookie，这样就相当于删除了。
                if(cookie.getName().equals(cookieName)){
                    return cookie.getValue();
                }
            }
        return "";
    }


    /**
     * 获得Cookie的值
     * @param cookieName
     * @return
     */
    public static String GetCookie(String cookieName)
    {
        return GetCookieValue(cookieName);
    }


    /**
     * 删除Cookie
     * @param cookieName
     */
    public static void RemoveCookie(String cookieName)
    {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        HttpServletResponse response = requestAttributes.getResponse();
        Cookie[] cookies = request.getCookies();
        Cookie cookiet = null;
        if (request != null)
            for(Cookie cookie : cookies){
                //如果找到同名cookie，就将value设置为null，将存活时间设置为0，再替换掉原cookie，这样就相当于删除了。
                if(cookie.getName().equals(cookieName)){
                    cookiet = cookie;
                    break;
                }
            }
        if (cookiet !=null && response != null)
        {
            for(Cookie cookie : cookies){
                //如果找到同名cookie，就将value设置为null，将存活时间设置为0，再替换掉原cookie，这样就相当于删除了。
                if(cookie.getName().equals(cookieName)){
                    cookie.setValue(null);
                    cookie.setMaxAge(0);
                    cookie.setPath("/");
                    response.addCookie(cookie);
                    break;
                }
            }
        }
    }


    /**
     * 设置cookie
     * @param cookieName
     * @param value
     * @param expires
     */
    public static void SetCookie(String cookieName,String value, Date expires)
    {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        HttpServletResponse response = requestAttributes.getResponse();
        Cookie[] cookies = null;

        if (request != null)cookies = request.getCookies();

        if (cookies!=null && cookieName !=null && expires != null && value !=null && response != null)
        {
            Date now = new Date();
            for(Cookie cookie : cookies){
                //如果找到同名cookie，就将value设置为null，将存活时间设置为0，再替换掉原cookie，这样就相当于删除了。
                if(cookie.getName().equals(cookieName)){
                    cookie.setValue(value);
                    cookie.setMaxAge((int)(now.getTime() - expires.getTime())/1000);//秒
                    cookie.setPath("/");
                    response.addCookie(cookie);
                    break;
                }
            }
        }

    }


    /**
     * 添加为Cookie.Values集合
     * @param cookieName
     * @param value
     * @param expires
     */
    public static void AddCookie(String cookieName, String value, Date expires)
    {

        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletResponse response = requestAttributes.getResponse();

        if (cookieName !=null && expires != null && value !=null && response != null)
        {
            Date now = new Date();
                //如果找到同名cookie，就将value设置为null，将存活时间设置为0，再替换掉原cookie，这样就相当于删除了。
            Cookie cookie = new Cookie(cookieName,value);
                 cookie.setMaxAge((int)(now.getTime() - expires.getTime())/1000);//秒
                    cookie.setPath("/");
                    response.addCookie(cookie);
        }
    }



}
