package com.liangjian11.ums.web.config.shiro;
import com.liangjian11.ums.credentials.RetryLimitHashedCredentialsMatcher;
import com.liangjian11.ums.web.shiro.JWTFilter;
import com.liangjian11.ums.web.shiro.JWTRealm;
//import com.avc.shiro.oauth2.realm.UserRealm;
import org.apache.shiro.authc.credential.HashedCredentialsMatcher;
//import org.apache.shiro.cache.ehcache.EhCacheManager;
import org.apache.shiro.mgt.DefaultSessionStorageEvaluator;
import org.apache.shiro.mgt.DefaultSubjectDAO;
import org.apache.shiro.spring.security.interceptor.AuthorizationAttributeSourceAdvisor;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.CookieRememberMeManager;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.apache.shiro.web.servlet.SimpleCookie;
import org.crazycake.shiro.RedisCacheManager;
import org.crazycake.shiro.RedisManager;
import org.crazycake.shiro.serializer.StringSerializer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.servlet.Filter;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

@Configuration
public class ShiroServerConfig {
	
	// 注意/r/n前不能有空格
	private static final String CRLF = "\r\n";

	private static final String MD5 = "md5";

	@Value("${jwt.token.active.time}")
	private static long activeTime =  12 * 60 * 60 * 1000;//毫秒

	@Value("${spring.redis.host}")
	private String host;
	@Value("${spring.redis.port}")
	private Integer port;
	@Value("${spring.redis.password}")
	private String password;

		
	@Bean(name="hashedCredMatcher")
	public HashedCredentialsMatcher createHashCredentialsMatcher() {
		RetryLimitHashedCredentialsMatcher hcm = new RetryLimitHashedCredentialsMatcher();
		hcm.setHashAlgorithmName(MD5);
		hcm.setStoredCredentialsHexEncoded(true);
//        hcm.setCacheManager(createEhcacheManager());
        hcm.setCacheManager(cacheManager());
		return hcm;
	}

//	@Bean(name="userAuthc")
//	public UserRealm getUserAuthcRealm() {
//		UserRealm uar = new UserRealm();
//		uar.setCredentialsMatcher(createHashCredentialsMatcher());
//		return uar;
//	}

//	@Bean(name="userAuthc")
	public JWTRealm getUserAuthcRealm(JWTRealm uar) {
//		JWTRealm uar = new JWTRealm();
//		uar.setCredentialsMatcher(createHashCredentialsMatcher());
		return uar;
	}
	
	@Bean(name="rememberCookies")
	public SimpleCookie createSimpleCookies() {
		SimpleCookie sc = new SimpleCookie("rememberMe");
		sc.setHttpOnly(true);
		sc.setMaxAge(5);
		return sc;
	}
	
	@Bean(name="cookieRemmberMananger")
	public CookieRememberMeManager createCookieRemmberMananger() {
		CookieRememberMeManager crm = new CookieRememberMeManager();
		crm.setCookie(createSimpleCookies());
		return crm;
	}
	
	/*public MemorySessionDAO createSessionDAO() {
		MemorySessionDAO sessionDao = new MemorySessionDAO();
		return null;
	}*/
	
//	@Bean(name="cacheManager")
//	public EhCacheManager createEhcacheManager() {
//		EhCacheManager cacheManager = new EhCacheManager();
//		String path = "classpath:ehcache.xml";
//		cacheManager.setCacheManagerConfigFile(path);
//		return cacheManager;
//	}

	/**
	 * shiro缓存管理器;
	 * 需要添加到securityManager中
	 * @return
	 */
	@Bean
	public RedisCacheManager cacheManager(){
		RedisCacheManager redisCacheManager = new RedisCacheManager();
		redisCacheManager.setRedisManager(redisManager());
		//redis中针对不同用户缓存
		redisCacheManager.setPrincipalIdFieldName("loginId");
		//用户权限信息缓存时间
		redisCacheManager.setExpire((int)activeTime/1000);
		redisCacheManager.setValueSerializer(new StringSerializer());
		return redisCacheManager;
	}

	@Bean
	public RedisManager redisManager(){
		RedisManager redisManager = new RedisManager();
		redisManager.setHost(this.host+":"+this.port);
//		redisManager.setPort(this.port);
		redisManager.setPassword(this.password);
		return redisManager;
	}
	
	@Bean(name="securityMananger")
	public DefaultWebSecurityManager createSecurityManager(@Autowired JWTRealm uar) {
		DefaultWebSecurityManager securityMananger = new DefaultWebSecurityManager();
		securityMananger.setRealm(getUserAuthcRealm(uar));
//		securityMananger.setCacheManager(createEhcacheManager());
		securityMananger.setCacheManager(cacheManager());
		securityMananger.setRememberMeManager(createCookieRemmberMananger());

        /*
         * 关闭shiro自带的session，详情见文档
         * http://shiro.apache.org/session-management.html#SessionManagement-StatelessApplications%28Sessionless%29
         */
        DefaultSubjectDAO subjectDAO = new DefaultSubjectDAO();
        DefaultSessionStorageEvaluator defaultSessionStorageEvaluator = new DefaultSessionStorageEvaluator();
        defaultSessionStorageEvaluator.setSessionStorageEnabled(false);
        subjectDAO.setSessionStorageEvaluator(defaultSessionStorageEvaluator);
		securityMananger.setSubjectDAO(subjectDAO);
		return securityMananger;
	}

	/**
	 * 自定义的shiro相关的Filter在这里做统一处理
	 * */
	public Map<String, Filter> createFilterChainMap() {
		Map<String, Filter> filters = new LinkedHashMap<>();
		return filters;
	}

	/**
	 * 读取ini文获得固定权限
	 * */
	public String loadFilterChainDefinitions() {
		StringBuffer sb = new StringBuffer();
		//	sb.append(getFixedAuthRule());//固定权限，采用读取配置文件
		/*
////         * 自定义url规则
////         * http://shiro.apache.org/web.html#urls-
////         */
//        Map<String, String> filterRuleMap = new HashMap<>();
//        filterRuleMap.put("/shiro/**", "jwt");
//        filterRuleMap.put("/shiro/auth/401", "anon");
//        //不拦截swagger
//        filterRuleMap.put("/swagger*/**", "anon");
//        filterRuleMap.put("/v2/**", "anon");
//        filterRuleMap.put("/webjars/**", "anon");
//        filterRuleMap.put("/**","anon");
//        factoryBean.setFilterChainDefinitionMap(filterRuleMap);

		sb.append("/shiro/auth/** = jwt").append(CRLF);//jwt
		sb.append("/shiro/auth/401 = anon").append(CRLF);
		sb.append("/shiro/auth/token = anon").append(CRLF);

		sb.append("/ums/oauth2 = anon").append(CRLF);
        sb.append("/ums/oauth2/login = authc").append(CRLF);//authc
        sb.append("/ums/oauth2/logout = logout").append(CRLF);
        sb.append("/ums/oauth2/authorize=anon").append(CRLF);
        sb.append("/ums/oauth2/accessToken=anon").append(CRLF);
        sb.append("/ums/oauth2/userInfo=anon").append(CRLF);
        sb.append("/ums/oauth2/** = jwt").append(CRLF);//jwt
		sb.append("/ums/** = jwt").append(CRLF);//jwt
//		sb.append("/ums/oauth/judge/permission = anon").append(CRLF);

		//Swagger的所有请求的资源和请求的地址都不需要拦截
		//不拦截swagger
		sb.append("/v2/** = anon").append(CRLF);
		sb.append("/swagger/** = anon").append(CRLF);
		sb.append("/v2/api-docs = anon").append(CRLF);
		sb.append("/swagger-ui.html = anon").append(CRLF);
		sb.append("/swagger-resources/** = anon").append(CRLF);
		sb.append("/webjars/** = anon").append(CRLF);
		sb.append("/favicon.ico = anon").append(CRLF);
		sb.append("/captcha.jpg = anon").append(CRLF);
		sb.append("/doc.html = anon").append(CRLF);
		sb.append("/csrf = anon").append(CRLF);

		sb.append("/** = anon").append(CRLF);

		return sb.toString();
	}
	

	
	 @Bean(name="shiroFilter")
	 public ShiroFilterFactoryBean createShiroSecurityFilterFactory(@Autowired JWTRealm uar) {
		 ShiroFilterFactoryBean shiroFilter = new ShiroFilterFactoryBean();

		 // 添加自己的过滤器并且取名为jwt
		 Map<String, Filter> filterMap = new HashMap<>();
		 filterMap.put("jwt", new JWTFilter());
		 shiroFilter.setFilters(filterMap);
		 //shiroFilter.setFilters(createFilterChainMap());

		 shiroFilter.setSecurityManager(createSecurityManager(uar));
		 shiroFilter.setLoginUrl("/login");


         //注意过滤器配置顺序 不能颠倒
  /*     filterChainDefinitionMap.put("/static/**", "anon");
         filterChainDefinitionMap.put("/templates/**", "anon");
         filterChainDefinitionMap.put("/g/unLogin", "anon");
         filterChainDefinitionMap.put("/g/login", "loginFtr");
         filterChainDefinitionMap.put("/g/logout", "authc");
         filterChainDefinitionMap.put("/g/unauthorized", "anon");
         filterChainDefinitionMap.put("/**", "permsFilter");*/

         shiroFilter.setFilterChainDefinitions(loadFilterChainDefinitions());

		 return shiroFilter;
	 }

    @Bean
    public AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor(@Qualifier("securityMananger") DefaultWebSecurityManager securityManager) {
        AuthorizationAttributeSourceAdvisor advisor = new AuthorizationAttributeSourceAdvisor();
        advisor.setSecurityManager(securityManager);
        return advisor;
    }
}
