package com.liangjian11.zuul.filter;

import com.liangjian11.zuul.configuration.CustomRouteLocator;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import org.apache.commons.lang3.StringUtils;
import org.apache.oltu.oauth2.common.message.types.GrantType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.netflix.zuul.filters.ZuulProperties;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * todo
 * Created by 陈东一
 * 2018/7/8 14:56
 */
@Component
public class AccessFilter extends ZuulFilter {

    @Autowired
    RestTemplate restTemplate;

    @Autowired
    CustomRouteLocator locator;

    @Override
    public Object run() {
        RequestContext ctx = RequestContext.getCurrentContext();
        HttpServletRequest request = ctx.getRequest();

        String token = request.getHeader("Sso-Token");
        if(StringUtils.isEmpty(token)){
            token = request.getParameter("Sso-Token");
        }

        System.out.println(String.format("%s %s %s %s", request.getMethod(), request.getRequestURL().toString(),request.getQueryString(),token));

        String requestURI = request.getRequestURI();
        
        //请求的是登陆地址
        if (requestURI.toLowerCase().contains("/ums/oauth2/accesstoken")) {
                String grant_type = request.getParameter("grant_type");// 获取请求的参数

                String client_id = request.getParameter("client_id");// 获取请求的参数
                String client_secret = request.getParameter("client_secret");// 获取请求的参数
                String applicationKey = request.getParameter("applicationKey");// 获取请求的参数

                String username = request.getParameter("username");// 获取请求的参数
                String password = request.getParameter("password");// 获取请求的参数

                if (client_id == null || client_secret == null|| applicationKey == null) {
                    ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                    ctx.setResponseStatusCode(200);// 返回错误码
                    ctx.setResponseBody("{\"success\":0,\"message\":\"client_id 或 client_secret 或 applicationKey 不能为空!\"}");// 返回错误内容
                    ctx.set("isSuccess", false);
                    setResponseCors(ctx);
                    return null;
                }

            if(grant_type!=null) {
                if (grant_type.equals(GrantType.PASSWORD.toString())) {
                    if (username == null || password == null) {
                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                        ctx.setResponseStatusCode(200);// 返回错误码
                        ctx.setResponseBody("{\"success\":0,\"message\":\"username 或 password 不能为空!\"}");// 返回错误内容
                        ctx.set("isSuccess", false);
                        setResponseCors(ctx);
                        return null;
                    }
                }else if(grant_type.equals(GrantType.CLIENT_CREDENTIALS.toString())){
                    //do nothing
                }
            }
        } else if (requestURI.toLowerCase().contains("/ums/oauth2/judge/token")) {
            if (token == null) {
                ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                ctx.setResponseStatusCode(200);// 返回错误码
                ctx.setResponseBody("{\"success\":0,\"message\":\"token 不能为空 !\"}");// 返回错误内容
                ctx.getResponse().setContentType("application/json;charset=UTF-8");
                ctx.set("isSuccess", false);
                setResponseCors(ctx);
                return null;
            }
        } else if (requestURI.toLowerCase().contains("/ums/oauth2/judge/permission")) {
            if (token == null) {
                ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                ctx.setResponseStatusCode(200);// 返回错误码
                ctx.setResponseBody("{\"result\":\"token 不能为空 !\"}");// 返回错误内容
                ctx.getResponse().setContentType("application/json;charset=UTF-8");
                ctx.set("isSuccess", false);
                setResponseCors(ctx);
                return null;
            }
        }
        else {

            String path = request.getServletPath();
            String[] paths = path.split("/");
            String pathMatch = "/" + paths[1] + "/**";
            Map<String, ZuulProperties.ZuulRoute> routes = locator.locateRoutesFromDB(true, pathMatch);

            ZuulProperties.ZuulRoute urlAddressR = routes.get(pathMatch);
            String serviceId = null;
            if(urlAddressR !=null )
              serviceId = urlAddressR.getServiceId();

            if (token == null) {
                ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                ctx.setResponseStatusCode(200);// 返回错误码
                ctx.setResponseBody("{\"success\":0,\"message\":\"token 不能为空 !\"}");// 返回错误内容
                ctx.getResponse().setContentType("application/json;charset=UTF-8");
                ctx.set("isSuccess", false);
                setResponseCors(ctx);
                return null;
            }

            String oauthService = "oauth2";
            if(paths[1].startsWith("oauth2") && serviceId != null )//不需要校验oauth2开头的测试 请求
                oauthService = serviceId;
            String url = "http://"+oauthService+"/ums/oauth2/judge/token";
//            String url = "http://192.168.50.32:8031/ums/oauth2/judge/token";


            MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
            headers.add("Content-Type", "application/json;charset=UTF-8");
            headers.add("Sso-Token", token);
            ResponseEntity<String> response = restTemplate.exchange(
                    url, HttpMethod.GET, new HttpEntity<Object>(headers),
                    String.class);

            ResultInfo body = JsonUtil.parseObject(response.getBody(), ResultInfo.class);

            @SuppressWarnings("unchecked")
            Map<String,Object> maps = (Map<String,Object>)body.getResult();
            String grant_type = (String) maps.get("grantType");
            if ( body == null || body.getMessageCode() != 200 || body.getMessageCode() == 105 ) {
                ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                ctx.setResponseStatusCode(200);// 返回错误码
                ctx.setResponseBody(response.getBody());// 返回错误内容
                ctx.getResponse().setContentType("application/json;charset=UTF-8");
                ctx.set("isSuccess", false);
                setResponseCors(ctx);
                return null;
            }

            /**
             *检查是否有权限调用这个接口
             * 暂时开启客户端模式权限配置
             */
            if( grant_type ==null ) {
                //passed do noting
            }else{
                if ( grant_type.equals(GrantType.PASSWORD.toString()) ||
//                        grant_type.equals(GrantType.AUTHORIZATION_CODE.toString())
                        grant_type.equals("authorizationCode")
                ) {
//                    if (path == null || paths.length < 2) {
//                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
//                        ctx.setResponseStatusCode(404);// 返回错误码
//                        ctx.setResponseBody("{\"success\":0,\"message\":\"" + "路径不合法" + "\"}");// 返回错误内容
//                        ctx.getResponse().setContentType("application/json;charset=UTF-8");
//                        ctx.set("isSuccess", false);
//                        return null;
//                    }
//
//
//                    String urlAuth = "http://oauth2/ums/oauth2/judge/permission";
//
//                    MultiValueMap<String, String> headersAuth = new LinkedMultiValueMap<String, String>();
//                    headersAuth.add("Content-Type", "application/x-www-form-urlencoded");
//                    headersAuth.add("Sso-Token", token);
//
//                    if (urlAddressR == null || urlAddressR.getUrl() == null) {
//                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
//                        ctx.setResponseStatusCode(404);// 返回错误码
//                        ctx.setResponseBody("{\"success\":0,\"message\":\"" + "配置路由的url不合法" + "\"}");// 返回错误内容
//                        ctx.getResponse().setContentType("application/json;charset=UTF-8");
//                        ctx.set("isSuccess", false);
//                        return null;
//                    }
//                    String urlAddress = urlAddressR.getUrl() + path;
//
//                    //如果给定一个MultiValueMap<String,String>，那么这个Map中的值将会被FormHttpMessageConverter以“application/x-www-form-urlencoded”的格式写到请求体中。
//                    MultiValueMap<String, String> mapAuth = new LinkedMultiValueMap<String, String>();
//                    mapAuth.add("urlAddress", urlAddress);
//
//                    //HttpEntity
//                    HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<MultiValueMap<String, String>>(mapAuth, headersAuth);
//
//                    ResponseEntity<String> responseAuth = restTemplate.postForEntity(urlAuth + "?urlAddress=" + urlAddress, requestEntity, String.class);
//
//                    ResultInfo bodyauth = JsonUtil.parseObject(responseAuth.getBody(), ResultInfo.class);
//
//                    @SuppressWarnings("unchecked")
//                    Map<String, Object> r = (Map<String, Object>) bodyauth.getResult();
//                    if (bodyauth.getMessageCode() != 200 || r == null || r.get("isAllowed").equals(false)) {
//                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
//                        ctx.setResponseStatusCode(401);// 返回错误码
////                String msg = r == null ||r.get("isAllowed").equals(false)?"授权失败":bodyauth.getMessage();
////                ctx.setResponseBody("{\"success\":0,\"result\":\"" + msg + "\"}");// 返回错误内容
//                        ctx.setResponseBody(responseAuth.getBody());
//                        ctx.getResponse().setContentType("application/json;charset=UTF-8");
//                        ctx.set("isSuccess", false);
//                        return null;
//                    }
                }else if (
//                        grant_type.equals(GrantType.CLIENT_CREDENTIALS.toString())
                        grant_type.equals("clientCredentials")
                ){

                    if (path == null || paths.length < 2) {
                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                        ctx.setResponseStatusCode(200);// 返回错误码
                        ctx.setResponseBody("{\"success\":0,\"message\":\"" + "路径不合法" + "\"}");// 返回错误内容
                        ctx.getResponse().setContentType("application/json;charset=UTF-8");
                        ctx.set("isSuccess", false);
                        setResponseCors(ctx);
                        return null;
                    }

                    if (urlAddressR == null || urlAddressR.getUrl() == null) {
                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                        ctx.setResponseStatusCode(200);// 返回错误码
                        ctx.setResponseBody("{\"success\":0,\"message\":\"" + "配置路由的url不合法" + "\"}");// 返回错误内容
                        ctx.getResponse().setContentType("application/json;charset=UTF-8");
                        ctx.set("isSuccess", false);
                        return null;
                    }



//                    String urlAddress = urlAddressR.getUrl() + path;
                    String urlAddress =  path;
                    String urlAuth = "http://"+oauthService+"/ums/oauth2/judge/permission";

                    MultiValueMap<String, String> headersAuth = new LinkedMultiValueMap<String, String>();
                    headersAuth.add("Content-Type", "application/x-www-form-urlencoded");
                    headersAuth.add("Sso-Token", token);

                    //如果给定一个MultiValueMap<String,String>，那么这个Map中的值将会被FormHttpMessageConverter以“application/x-www-form-urlencoded”的格式写到请求体中。
                    MultiValueMap<String, String> mapAuth = new LinkedMultiValueMap<String, String>();
                    mapAuth.add("urlAddress", urlAddress);

                    //HttpEntity
                    HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<MultiValueMap<String, String>>(mapAuth, headersAuth);
                    ResponseEntity<String> responseAuth = restTemplate.postForEntity(urlAuth + "?urlAddress=" + urlAddress, requestEntity, String.class);

                    ResultInfo bodyauth = JsonUtil.parseObject(responseAuth.getBody(), ResultInfo.class);

                    @SuppressWarnings("unchecked")
                    Map<String, Object> r = (Map<String, Object>) bodyauth.getResult();
                    if (bodyauth.getMessageCode() != 200 || r == null || r.get("isAllowed").equals(false)) {
                        ctx.setSendZuulResponse(false);// 过滤该请求，不对其进行路由
                        ctx.setResponseStatusCode(200);// 返回码
                        String msg = r != null  && bodyauth.getSuccess() == 0 && !StringUtils.isEmpty(bodyauth.getMessage()) ?bodyauth.getMessage():"没有授权，访问失败";
                        ctx.setResponseBody("{\"success\":0,\"message\":\"" + msg + "\"}");// 返回错误内容
//                        ctx.setResponseBody(responseAuth.getBody());
                        ctx.getResponse().setContentType("application/json;charset=UTF-8");
                        ctx.set("isSuccess", false);
                        setResponseCors(ctx);
                        return null;
                    }

                }//end CLIENT_CREDENTIALS
            }
        }

        ctx.setSendZuulResponse(true);// 对该请求进行路由
        ctx.setResponseStatusCode(200);
        ctx.set("isSuccess", true);// 设值，让下一个Filter看到上一个Filter的状态
        return null;
    }
    
    @Override
    public boolean shouldFilter() {
        RequestContext ctx = RequestContext.getCurrentContext();
        HttpServletRequest request = ctx.getRequest();
        //过滤各种POST请求
        if(request.getMethod().equals(RequestMethod.OPTIONS.name())){
            return false;
        }

        return true;// 是否执行该过滤器，此处为true，说明需要过滤
    }
    
    @Override
    public int filterOrder() {
        return 0;// 优先级为0，数字越大，优先级越低
    }
    
    @Override
    public String filterType() {
        return "pre";// 前置过滤器
    }

    public static void setResponseCors(RequestContext ctx){
        //添加跨域CORS
        ctx.getResponse().setHeader("Access-Control-Allow-Origin", "*");
        ctx.getResponse().setHeader("Access-Control-Allow-Headers",
                "DNT,X-Mx-ReqToken,Keep-Alive,User-Agent,X-Requested-With,If-Modified-Since,Cache-Control,Content-Type," +
                        "bird-idempotency,Sso-Token");
        ctx.getResponse().setHeader("Access-Control-Allow-Methods", "GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, PATCH");
        ctx.getResponse().setHeader("Allow", "GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, PATCH");
    }
    
}
